/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2009 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: AccountingDatasourceBL.java 2688 2010-11-04 16:47:10Z friedj $ 
 * 
 */

package com.trackplus.plugin.report.accounting;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import com.aurel.track.dao.AccountDAO;
import com.aurel.track.dao.CostCenterDAO;
import com.aurel.track.dao.DAOFactory;
import com.aurel.track.dao.DepartmentDAO;
import com.aurel.track.dao.PersonDAO;
import com.aurel.track.dao.ProjectDAO;
import com.aurel.track.resources.LocalizeUtil;
import com.aurel.track.util.IntegerStringBean;
import com.trackplus.plugin.report.accounting.AccountingDatasourceBase.OperationMode;
import com.trackplus.plugin.report.accounting.AccountingDatasourceBase.TimeInterval;

public class AccountingDatasourceBL {
		
	private static PersonDAO personDAO = DAOFactory.getFactory().getPersonDAO();
	private static ProjectDAO projectDAO = DAOFactory.getFactory().getProjectDAO();
	private static CostCenterDAO costCenterDAO = DAOFactory.getFactory().getCostCenterDAO();
	private static AccountDAO accountDAO = DAOFactory.getFactory().getAccountDAO();
	private static DepartmentDAO departmentDAO = DAOFactory.getFactory().getDepartmentDAO();
	
	/**
     * loads the persons which belong to the specified departments 
     * @param selectedDepartments
     * @return a list of LabelValueBeans
     */
    public static List getPersonsOptionsListByDepartments(Integer[] selectedDepartments) {
    	if (selectedDepartments == null || selectedDepartments.length==0) {
    		return getAllPersonsOptionsList();
    	}
    	return personDAO.loadByDepartments(selectedDepartments); 	       	                           	        
    }
    
    /**
     * loads all persons
     * @return a list of LabelValueBeans
     */
    public static List getAllPersonsOptionsList() {    	    	
    	return personDAO.loadPersons1() ;    	        
    }
    
    public static List getAllCostcentersOptionsList() {
    	return costCenterDAO.loadAll();
    }
    
    public static List getAllAccountsOptionsList() {
    	return accountDAO.loadAll();    	     
    }
    
    /**
     * loads the accounts which belong to the specified costcenters 
     * @param selectedCostcenters
     * @return a list of LabelValueBeans
     */
    public static List getAccountsOptionsListByCostcenters(Integer[] selectedCostcenters) {    	
    	if (selectedCostcenters == null || selectedCostcenters.length==0) {
    		return getAllAccountsOptionsList();
    	}
    	return accountDAO.loadByCostcenters(selectedCostcenters);     
    }
    
    /**
     * loads all departments
     * @return a list of LabelValueBeans
     */
    public static List getAllDepartmentsOptionsList() {
    	return departmentDAO.loadAll1();    	        
    }
    
    /**
     * loads the projects where the user has view hours/costs right 
     * @param userKey
     * @return a list of LabelValueBeans
     */
    public static List getProjectsOptionsListByUserAsOwn(Integer personID) {
    	return projectDAO.loadUsedProjects(personID);    	
    }
    
    /**
     * loads the projects where the user has view hours/costs right 
     * @param userKey
     * @return a list of LabelValueBeans
     */
    public static List getProjectsOptionsListByUserAsManager(Integer personID) {
    	return projectDAO.loadProjectsWithViewAllExpensesRight(personID);  	    	
    }    
	
	/**
	 * Localize the time intervals
	 * @param locale
	 * @param bundleName
	 * @return
	 */
    public static List getLocalizedTimeIntervals(Locale locale, String bundleName) {
		List localizedTimeIntervals = new ArrayList();
		String prefix = "accountingReportConfig.prompt.timeInterval.";
		localizedTimeIntervals.add(new IntegerStringBean(LocalizeUtil.getLocalizedText(
				prefix+TimeInterval.CURRENTMONTH, locale, bundleName), 
				new Integer(TimeInterval.CURRENTMONTH))); 
		localizedTimeIntervals.add(new IntegerStringBean(LocalizeUtil.getLocalizedText(
				prefix+TimeInterval.LASTMONTH, locale, bundleName), 
				new Integer (TimeInterval.LASTMONTH)));
		localizedTimeIntervals.add(new IntegerStringBean(LocalizeUtil.getLocalizedText(
				prefix+TimeInterval.FROMTO, locale, bundleName), 
				new Integer(TimeInterval.FROMTO)));
		return localizedTimeIntervals;
	}
	

	/**
	 * Localize the time intervals
	 * @param locale
	 * @param bundleName
	 * @return
	 */
    public static List getAccountingReportOperationModes(Locale locale, String bundleName) {
		List reportOperationModes = new ArrayList();
		String prefix = "accountingReportConfig.prompt.operationMode.";
		reportOperationModes.add(new IntegerStringBean(LocalizeUtil.getLocalizedText(
				prefix+OperationMode.PERSONALOVERVIEW, locale, bundleName), 
				new Integer(OperationMode.PERSONALOVERVIEW))); 
		reportOperationModes.add(new IntegerStringBean(LocalizeUtil.getLocalizedText(
				prefix+OperationMode.MANAGEROVERVIEW, locale, bundleName), 
				new Integer (OperationMode.MANAGEROVERVIEW)));		
		return reportOperationModes;
	}
}
