/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2009 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: AccountingDatasourceGeneral.java 2688 2010-11-04 16:47:10Z friedj $ 
 * 
 */

package com.trackplus.plugin.report.accounting;

import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.dom.CDATASection;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

import com.aurel.track.beans.TAccountBean;
import com.aurel.track.beans.TCostBean;
import com.aurel.track.beans.TCostCenterBean;
import com.aurel.track.beans.TListTypeBean;
import com.aurel.track.beans.TPersonBean;
import com.aurel.track.beans.TProjectBean;
import com.aurel.track.beans.TProjectCategoryBean;
import com.aurel.track.beans.TStateBean;
import com.aurel.track.beans.TWorkItemBean;
import com.aurel.track.dao.AccountDAO;
import com.aurel.track.dao.CostCenterDAO;
import com.aurel.track.dao.CostDAO;
import com.aurel.track.dao.DAOFactory;
import com.aurel.track.dao.IssueTypeDAO;
import com.aurel.track.dao.PersonDAO;
import com.aurel.track.dao.ProjectDAO;
import com.aurel.track.dao.StateDAO;
import com.aurel.track.dao.SubprojectDAO;
import com.aurel.track.dao.WorkItemDAO;
import com.aurel.track.plugin.DatasourceDescriptor;
import com.aurel.track.report.datasource.DatasourceCommonBL;
import com.aurel.track.report.export.ReportBeansToXML;
import com.aurel.track.util.DateTimeUtils;
import com.aurel.track.util.GeneralUtils;
import com.aurel.track.util.EmailHandling.Html2Text;
import com.aurel.track.util.numberFormatter.DoubleWithDecimalDigitsNumberFormatUtil;

public class AccountingDatasourceGeneral extends AccountingDatasourceBase {

	private static Logger logger = Logger.getLogger(AccountingDatasourceGeneral.class);
	
	private static CostDAO costDAO = DAOFactory.getFactory().getCostDAO();
	private static WorkItemDAO workItemDAO = DAOFactory.getFactory().getWorkItemDAO();
	private static PersonDAO personDAO = DAOFactory.getFactory().getPersonDAO();
	private static ProjectDAO projectDAO = DAOFactory.getFactory().getProjectDAO();
	private static SubprojectDAO subprojectDAO = DAOFactory.getFactory().getSubprojectDAO();
	private static CostCenterDAO costCenterDAO = DAOFactory.getFactory().getCostCenterDAO();
	private static AccountDAO accountDAO = DAOFactory.getFactory().getAccountDAO();
	private static StateDAO stateDAO = DAOFactory.getFactory().getStateDAO();
	private static IssueTypeDAO issueTypeDAO = DAOFactory.getFactory().getIssueTypeDAO();
	
	public Object getDatasource(Map parameters,
			DatasourceDescriptor datasourceDescriptor, TPersonBean personBean,
			Locale locale) {
		parameters.put(PARAMETER_NAME.locale, locale);
		parameters.put(PARAMETER_NAME.bundleName, datasourceDescriptor.getBundleName());
		AccountingReportConfigStruct configStruct = getConfigData(parameters, personBean.getObjectID());				
		List<Map<String, String>> expensesList = getExpensesList(configStruct);		
		return convertToDOM(expensesList, getFieldNameList(), personBean.getFullName(), locale);
	}

	public void serializeDatasource(OutputStream outputStream,
			Object datasource) {
		ReportBeansToXML.convertToXml(outputStream, (Document)datasource);
	}

	public boolean implementSerialization() {
		return true;
	}
	
	private Document convertToDOM(List<Map<String, String>> expensesList, List<String> fieldNames, String personName, Locale locale) {
		Document dom = null;
		try{
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
			DocumentBuilder builder = factory.newDocumentBuilder ();
			dom = builder.newDocument ();	
		} catch (FactoryConfigurationError e){
			logger.error ("Creating the DOM document failed with FactoryConfigurationError:" + e.getMessage());
			return null;
		} catch (ParserConfigurationException e){
			logger.error ("Creating the DOM document failed with ParserConfigurationException: " + e.getMessage());
			return null;
		}
		Element root = dom.createElement ("expense-report");
        Element createdBy =createDomElement("createdBy",personName, false, dom);
        if (createdBy!=null) {
        	root.appendChild(createdBy);
        }
        if (expensesList!=null) {
        	Iterator<Map<String, String>> itrExpenses = expensesList.iterator();
        	while (itrExpenses.hasNext()) {
				Map<String, String> expenseMap = (Map<String, String>) itrExpenses.next();
				Element expenseElement = createDomElement("expense", null, false, dom);
				root.appendChild(expenseElement);
				Iterator<String> itrFieldNames = fieldNames.iterator();
				while (itrFieldNames.hasNext()) {
					String fieldName = (String) itrFieldNames.next();
					expenseElement.appendChild(createDomElement(fieldName, expenseMap.get(fieldName), false, dom));
				}
			}
        }
        dom.appendChild(root);
		return dom;
	 }
	
	 /**
	 * Create a dom element with text content and attributes
	 * @param elementName
	 * @param textContent
	 * @param attributeValues
	 * @param dom
	 * @return
	 */
	private static Element createDomElement(String elementName, String textContent, boolean needCDATA,
			/*Map<String, String> attributeValues,*/ Document dom) {
		//create the main element
		Element element = null;
		try {
			element = dom.createElement(elementName);
		} catch (DOMException e) {
			logger.warn("Creating an XML node with the element name " + elementName + " failed with " + e);
		}
		if (element==null) {
			return null;
		}
		
		//create the text element
		if (textContent!=null) {						
			if (needCDATA) {
				CDATASection cdataText = dom.createCDATASection(Html2Text.stripNonValidXMLCharacters(textContent));
				element.appendChild(cdataText);				
			} else {
				Text textElement;				
				try {
					textElement = dom.createTextNode(textContent);				
				} catch (DOMException e) {
					logger.info("Creating the text node for the element " + elementName + " and text " + textContent + " failed with " + e);
					textElement = dom.createTextNode("");				
				}			
				//append the text to the element
				element.appendChild(textElement);
			}								
		}				
		return element;
	}
	
	private static List<Map<String, String>> getExpensesList(AccountingReportConfigStruct configStruct) {
		List<Map<String, String>> expensesList = new ArrayList<Map<String,String>>();
		Date from = configStruct.getDateFrom();
		Date to = configStruct.getDateTo();
		List<Integer> selectedProjectsAndReleases = configStruct.getSelectedProjects();
		List<Integer> selecetedExplicitProjects = new ArrayList<Integer>();
		List<Integer> selecetedImplicitProjects = new ArrayList<Integer>();
		List<Integer> selectedReleases = new ArrayList<Integer>();		
		DatasourceCommonBL.getProjectsAndReleasesIDs(selectedProjectsAndReleases, 
				selecetedExplicitProjects, selecetedImplicitProjects, selectedReleases);
		List<Integer> allProjectsList = new ArrayList<Integer>();
		allProjectsList.addAll(selecetedExplicitProjects);
		allProjectsList.addAll(selecetedImplicitProjects);
		int[] selectedPersons = configStruct.getSelectedPersons();
		int[] selectedAccounts = configStruct.getSelectedAccounts();
		List<TCostBean> costBeansList = costDAO.loadExpenses(from, to, selecetedExplicitProjects, selectedReleases, selectedPersons, selectedAccounts);
		if (costBeansList!=null && costBeansList.size()>0) {
			Map<Integer, TWorkItemBean> workItemBeanMap = GeneralUtils.createMapFromList(
					workItemDAO.getWorkItemWithExpense(from, to, selecetedExplicitProjects, selectedReleases, selectedPersons, selectedAccounts));
			Map<Integer, TPersonBean> personBeanMap = GeneralUtils.createMapFromList(
					personDAO.getWorkItemWithExpense(from, to, selecetedExplicitProjects, selectedReleases, selectedPersons, selectedAccounts));
			Map<Integer, TAccountBean> accountBeanMap;
			if (selectedAccounts==null || selectedAccounts.length==0) {
				accountBeanMap = GeneralUtils.createMapFromList(accountDAO.loadAll());
			} else {
				accountBeanMap = GeneralUtils.createMapFromList(accountDAO.loadByKeys(GeneralUtils.createListFromIntArr(selectedAccounts)));
			}			
			Map<Integer, TCostCenterBean> costcenterBeanMap = GeneralUtils.createMapFromList(costCenterDAO.loadAll());
			List<TProjectBean> projectBeansList = projectDAO.loadByProjectIDs(allProjectsList);
			Map<Integer, TProjectBean> projectBeansMap = GeneralUtils.createMapFromList(projectBeansList);
			
			Map<Integer, TProjectCategoryBean> subprojectBeansMap = GeneralUtils.createMapFromList(
					subprojectDAO.loadByProjectKeys(GeneralUtils.createIntegerListFromBeanList(projectBeansList)));
			
			Map<Integer, TStateBean> statusBeanMap = GeneralUtils.createMapFromList(stateDAO.loadAll());
			Map<Integer, TListTypeBean> issueTypeBeanMap = GeneralUtils.createMapFromList(issueTypeDAO.loadAll1());
			
			Iterator<TCostBean> itrCostBean = costBeansList.iterator();
			while (itrCostBean.hasNext()) {
				TCostBean costBean = (TCostBean) itrCostBean.next();
				Map<String, String> expenseMap = new HashMap<String, String>();										
				//locale is english because we need the iso format
				expenseMap.put(ACCOUNTING_FIELDNAMES.HOURS,  
						DoubleWithDecimalDigitsNumberFormatUtil.getInstance(2).formatGUI(costBean.getHours(), Locale.ENGLISH));
				expenseMap.put(ACCOUNTING_FIELDNAMES.COST, 
						DoubleWithDecimalDigitsNumberFormatUtil.getInstance(2).formatGUI(costBean.getCost(), Locale.ENGLISH));
				expenseMap.put(ACCOUNTING_FIELDNAMES.SUBJECT, costBean.getSubject());
				expenseMap.put(ACCOUNTING_FIELDNAMES.EFFORTDATE, DateTimeUtils.formatISODateTime(costBean.getEffortdate()));
				expenseMap.put(ACCOUNTING_FIELDNAMES.DESCRIPTION, costBean.getDescription());
				expenseMap.put(ACCOUNTING_FIELDNAMES.LASTEDIT, DateTimeUtils.formatISODateTime(costBean.getLastEdit()));
				
				if (costBean.getAccount()!=null) {
					expenseMap.put(ACCOUNTING_FIELDNAMES.ACCOUNT, costBean.getAccount().toString());
					TAccountBean accountBean = accountBeanMap.get(costBean.getAccount());
					if (accountBean!=null) {
						expenseMap.put(ACCOUNTING_FIELDNAMES.ACCOUNTNAME, accountBean.getAccountName());
						expenseMap.put(ACCOUNTING_FIELDNAMES.ACCOUNTNUMBER, accountBean.getAccountNumber());
						if (accountBean.getStatus()!=null) {
							expenseMap.put(ACCOUNTING_FIELDNAMES.ACCOUNTSTATUS, accountBean.getStatus().toString());
						}
						if (accountBean.getCostCenter()!=null) {
							expenseMap.put(ACCOUNTING_FIELDNAMES.COSTCENTER, accountBean.getCostCenter().toString());
							TCostCenterBean costCenterBean = costcenterBeanMap.get(accountBean.getCostCenter());
							if (costCenterBean!=null) {								
								expenseMap.put(ACCOUNTING_FIELDNAMES.COSTCENTERNAME, costCenterBean.getCostcenterName());
								expenseMap.put(ACCOUNTING_FIELDNAMES.COSTCENTERNUMBER, costCenterBean.getCostcenterNumber());
							}
						}
					}
				}
				
				if (costBean.getPerson()!=null) {
					expenseMap.put(ACCOUNTING_FIELDNAMES.PERSON, costBean.getPerson().toString());
					TPersonBean personBean = personBeanMap.get(costBean.getPerson());
					if (personBean!=null) {
						expenseMap.put(ACCOUNTING_FIELDNAMES.PERSONNAME, personBean.getFullName());
					}
				}
				
				if (costBean.getWorkitem()!=null) {
					expenseMap.put(ACCOUNTING_FIELDNAMES.WORKITEM, costBean.getWorkitem().toString());
					TWorkItemBean workItemBean = workItemBeanMap.get(costBean.getWorkitem());
					if (workItemBean!=null) {
						expenseMap.put(ACCOUNTING_FIELDNAMES.WORKITEMTITLE, workItemBean.getSynopsis());
						if (workItemBean.getProjectCategoryID()!=null) {
							expenseMap.put(ACCOUNTING_FIELDNAMES.SUBPROJECT, workItemBean.getProjectCategoryID().toString());
							TProjectCategoryBean subprojectBean = subprojectBeansMap.get(workItemBean.getProjectCategoryID());
							if (subprojectBean!=null) {								
								expenseMap.put(ACCOUNTING_FIELDNAMES.SUBPROJECTLABEL, subprojectBean.getLabel());
								TProjectBean projectBean = projectBeansMap.get(subprojectBean.getProjectID());
								if (projectBean!=null) {
									expenseMap.put(ACCOUNTING_FIELDNAMES.PROJECT, projectBean.getObjectID().toString());
									expenseMap.put(ACCOUNTING_FIELDNAMES.PROJECTLABEL, projectBean.getLabel());
									expenseMap.put(ACCOUNTING_FIELDNAMES.CURRENCYSYMBOL, projectBean.getCurrencySymbol());
									expenseMap.put(ACCOUNTING_FIELDNAMES.CURRENCYNAME, projectBean.getCurrencyName());									
								}
							}
						}
						
						if (workItemBean.getStateID()!=null) {
							expenseMap.put(ACCOUNTING_FIELDNAMES.STATUS, workItemBean.getStateID().toString());
							TStateBean stateBean = statusBeanMap.get(workItemBean.getStateID());
							if (stateBean!=null) {
								expenseMap.put(ACCOUNTING_FIELDNAMES.STATUSLABEL, stateBean.getLabel());
							}
						}
						if (workItemBean.getListTypeID()!=null) {
							expenseMap.put(ACCOUNTING_FIELDNAMES.ISSUETYPE, workItemBean.getListTypeID().toString());
							TListTypeBean issueTypeBean = issueTypeBeanMap.get(workItemBean.getListTypeID());
							if (issueTypeBean!=null) {
								expenseMap.put(ACCOUNTING_FIELDNAMES.ISSUETYPELABEL, issueTypeBean.getLabel());		
							}
						}						
					}
				}
				expensesList.add(expenseMap);		
			}
		}
		return expensesList;		
	}
	
	private static List<String> getFieldNameList() {
		List<String> fieldNames = new ArrayList<String>();
		fieldNames.add(ACCOUNTING_FIELDNAMES.ACCOUNT);
		fieldNames.add(ACCOUNTING_FIELDNAMES.PERSON);
		fieldNames.add(ACCOUNTING_FIELDNAMES.WORKITEM);
		fieldNames.add(ACCOUNTING_FIELDNAMES.HOURS);
		fieldNames.add(ACCOUNTING_FIELDNAMES.COST);
		fieldNames.add(ACCOUNTING_FIELDNAMES.SUBJECT);
		fieldNames.add(ACCOUNTING_FIELDNAMES.EFFORTDATE);
		fieldNames.add(ACCOUNTING_FIELDNAMES.DESCRIPTION);
		fieldNames.add(ACCOUNTING_FIELDNAMES.LASTEDIT);
		
		fieldNames.add(ACCOUNTING_FIELDNAMES.ACCOUNTNAME);
		fieldNames.add(ACCOUNTING_FIELDNAMES.ACCOUNTNUMBER);
		fieldNames.add(ACCOUNTING_FIELDNAMES.STATUS);
		fieldNames.add(ACCOUNTING_FIELDNAMES.COSTCENTER);
		
		fieldNames.add(ACCOUNTING_FIELDNAMES.COSTCENTERNAME);
		fieldNames.add(ACCOUNTING_FIELDNAMES.COSTCENTERNUMBER);
		
		fieldNames.add(ACCOUNTING_FIELDNAMES.PERSONNAME);		
		
		fieldNames.add(ACCOUNTING_FIELDNAMES.WORKITEMTITLE);
		fieldNames.add(ACCOUNTING_FIELDNAMES.STATUS);
		fieldNames.add(ACCOUNTING_FIELDNAMES.ISSUETYPE);
		
		fieldNames.add(ACCOUNTING_FIELDNAMES.SUBPROJECT);
		fieldNames.add(ACCOUNTING_FIELDNAMES.SUBPROJECTLABEL);
		
		fieldNames.add(ACCOUNTING_FIELDNAMES.PROJECT);
		fieldNames.add(ACCOUNTING_FIELDNAMES.PROJECTLABEL);
		fieldNames.add(ACCOUNTING_FIELDNAMES.CURRENCYSYMBOL);
		fieldNames.add(ACCOUNTING_FIELDNAMES.CURRENCYNAME);
		
		fieldNames.add(ACCOUNTING_FIELDNAMES.STATUSLABEL);
		
		fieldNames.add(ACCOUNTING_FIELDNAMES.ISSUETYPELABEL);
		
		return fieldNames;
	}
	
	private interface ACCOUNTING_FIELDNAMES {
		String ACCOUNT = "accountID";
		String PERSON = "personID";
		String WORKITEM = "workItemID";
		String HOURS = "hours";
		String COST = "cost";
		String SUBJECT = "subject";
		String EFFORTDATE = "effortDate";
		String DESCRIPTION = "description";
		String LASTEDIT = "lastEdit";
		
		String ACCOUNTNAME = "accountName";
		String ACCOUNTNUMBER = "accountNumber";
		String ACCOUNTSTATUS = "accountStatus";
		
		String COSTCENTER = "costcenterID";
		String COSTCENTERNAME = "costcenterName";
		String COSTCENTERNUMBER = "costcenterNumber";
		
		String PERSONNAME = "personName";		
		
		String WORKITEMTITLE = "workItemTitle";
		String STATUS = "status";
		String ISSUETYPE = "issueType";
		
		String SUBPROJECT = "subprojectID";
		String SUBPROJECTLABEL = "subprojectLabel";
		
		String PROJECT = "projectID";		
		String PROJECTLABEL= "projectLabel";
		String CURRENCYSYMBOL = "currencySymbol";
		String CURRENCYNAME = "currencyName";
		
		String STATUSLABEL= "statusLabel";
		
		String ISSUETYPELABEL = "issueTypeLabel";				
	}
		
}
