package com.trackplus.plugin.report.earnedValue;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.aurel.track.beans.TBudgetBean;
import com.aurel.track.beans.TCostBean;
import com.aurel.track.beans.TProjectBean;
import com.aurel.track.beans.TProjectTypeBean;
import com.aurel.track.beans.TStateBean;
import com.aurel.track.beans.TWorkItemBean;
import com.aurel.track.dao.DAOFactory;
import com.aurel.track.dao.ProjectDAO;
import com.aurel.track.dao.ProjectTypeDAO;
import com.aurel.track.dao.StateDAO;
import com.aurel.track.fieldType.constants.SystemFields;
import com.aurel.track.item.budgetCost.AccountingBL;
import com.aurel.track.item.history.HistoryLoaderBL;
import com.aurel.track.item.history.HistoryValues;
import com.aurel.track.report.datasource.DatasourceCommonBL;
import com.aurel.track.report.execute.ReportBeanWithHistory;
import com.aurel.track.resources.LocalizeUtil;
import com.aurel.track.util.DateTimeUtils;
import com.aurel.track.util.GeneralUtils;



public class EarnedValueBL {
	private static Logger logger = Logger.getLogger(EarnedValueBL.class);		
	private static ProjectDAO projectDAO = DAOFactory.getFactory().getProjectDAO();
	private static ProjectTypeDAO projectTypeDAO = DAOFactory.getFactory().getProjectTypeDAO();
	protected static StateDAO stateDAO = DAOFactory.getFactory().getStateDAO();
	
	static List getStatuses(Locale locale) {
		return LocalizeUtil.localizeDropDownList(stateDAO.loadAll(), locale);
	}
	
	static List<TStateBean> loadClosedStates() {
		return stateDAO.loadByStateFlag(TStateBean.STATEFLAGS.CLOSED);
	}
	
	static Set<Integer> prefilterReportBeansGetProjectIDs(List<ReportBeanWithHistory> reportBeanWithHistoryList, 
			boolean isTime, Date dateFrom, Date dateTo) {
		Set<Integer> projectIDs = new HashSet<Integer>();
		if (reportBeanWithHistoryList!=null) {
			Iterator<ReportBeanWithHistory> iterator = reportBeanWithHistoryList.iterator();
			while (iterator.hasNext()) {
				ReportBeanWithHistory reportBeanWithHistory = (ReportBeanWithHistory) iterator.next();
				List<TBudgetBean> budgetList = reportBeanWithHistory.getBudgetHistory();
				if (budgetList==null || budgetList.size()==0) {
					//no budget exists
					iterator.remove();
					continue;
				} else {
					//get the first entry which should be the most recent one
					TBudgetBean budgetBean = budgetList.get(0);
					if (isTime) {						
						Double effortValue = budgetBean.getEstimatedHours();
						if (effortValue==null || effortValue.doubleValue()==0.0) {
							//no planned effort
							iterator.remove();
							continue;
						}						
					} else {
						Double costValue = budgetBean.getEstimatedCost();
						if (costValue==null || costValue.doubleValue()==0.0) {
							//no planned cost
							iterator.remove();
							continue;
						}
					}
				}
				TWorkItemBean workItemBean = reportBeanWithHistory.getWorkItemBean();
				Date startDate = workItemBean.getStartDate();
				Date endDate = workItemBean.getEndDate();
				if (dateFrom!=null && startDate!=null && startDate.before(dateFrom)) {
					//starts before dateFrom
					iterator.remove();
					continue;
				}
				if (dateTo!=null && endDate!=null && endDate.after(dateTo)) {
					//ends after dateTo
					iterator.remove();
					continue;
				}
				projectIDs.add(workItemBean.getProjectID());
			}
		}
		return projectIDs;
	}
	
	static Date getMinMaxStartEndDate(List<ReportBeanWithHistory> reportBeanWithHistoryList, boolean isStartDate) {		
		Date minMaxDate = null;
		if (reportBeanWithHistoryList!=null) {
			Iterator<ReportBeanWithHistory> iterator = reportBeanWithHistoryList.iterator();
			while (iterator.hasNext()) {
				ReportBeanWithHistory reportBeanWithHistory = (ReportBeanWithHistory) iterator.next();				
				TWorkItemBean workItemBean = reportBeanWithHistory.getWorkItemBean();				
				Date currentDate;
				if (isStartDate) {
					currentDate = workItemBean.getStartDate();
				} else {
					currentDate = workItemBean.getEndDate();
				}
				if (currentDate!=null) {
					if (minMaxDate == null) {
						minMaxDate = currentDate;
					} else {
						if (isStartDate) {
							if (currentDate.before(minMaxDate)) {
								minMaxDate = currentDate;
							}
						} else {
							if (currentDate.after(minMaxDate)) {
								minMaxDate = currentDate;
							}
						}
					}
				}				
			}
		}
		return minMaxDate;
	}
	
	static Map<Integer, Double> getHoursPerWorkingDayMap(Set<Integer> projectIDs) {
		Map<Integer, Double> projectHoursPerWorkingDayMap = new HashMap<Integer, Double>();
		List<TProjectBean> projectBeansList = projectDAO.loadByProjectIDs(GeneralUtils.createListFromSet(projectIDs));
		Set<Integer> projectTypeSet = new HashSet<Integer>();
		if (projectBeansList!=null) {
			Iterator<TProjectBean> itrProject = projectBeansList.iterator();
			while (itrProject.hasNext()) {
				TProjectBean projectBean = (TProjectBean) itrProject.next();
				if (projectBean.getHoursPerWorkDay()!=null && projectBean.getHoursPerWorkDay().doubleValue() > 0.0) {
					//specifided at project level
					projectHoursPerWorkingDayMap.put(projectBean.getObjectID(), projectBean.getHoursPerWorkDay());
					itrProject.remove();
				} else {
					//not specified at project level, gather the projectTypes
					projectTypeSet.add(projectBean.getProjectType());
				}								
			}
			if (projectTypeSet.size()>0) {
				Map<Integer, Double> projectTypeHoursPerWorkingDayMap = new HashMap<Integer, Double>();
				List<TProjectTypeBean> projectTypes = projectTypeDAO.loadByProjectTypeIDs(
						GeneralUtils.createListFromSet(projectTypeSet));
				Iterator<TProjectTypeBean> itrProjectType = projectTypes.iterator();
				while (itrProjectType.hasNext()) {
					TProjectTypeBean projectTypeBean = itrProjectType.next();
					if (projectTypeBean.getHoursPerWorkDay()!=null && projectTypeBean.getHoursPerWorkDay().doubleValue() > 0.0) {
						//specifided at projectType level
						projectTypeHoursPerWorkingDayMap.put(projectTypeBean.getObjectID(), projectTypeBean.getHoursPerWorkDay());
					}
				}
				//loop through the projects again
				itrProject = projectBeansList.iterator();
				while (itrProject.hasNext()) {
					TProjectBean projectBean = (TProjectBean) itrProject.next();
					Integer projectType = projectBean.getProjectType();
					if (projectTypeHoursPerWorkingDayMap.containsKey(projectType)) {
						projectHoursPerWorkingDayMap.put(projectBean.getObjectID(), projectTypeHoursPerWorkingDayMap.get(projectType));
					} else {
						projectHoursPerWorkingDayMap.put(projectBean.getObjectID(), new Double(AccountingBL.DEFAULTHOURSPERWORKINGDAY));
					}
				}
			}
		}
		return projectHoursPerWorkingDayMap;
	}

	
		  
   	static SortedMap<Integer, SortedMap<Integer, Double>> calculatePlannedValues(
   			List<ReportBeanWithHistory> reportBeanWithHistoryList, 
   			boolean isTime, Date minStartDate, Date maxEndDate, Integer timeInterval, Map<Integer, Double> hoursPerWorkingDayMap) {
   		SortedMap<Integer, SortedMap<Integer, Double>> yearToIntervalToPlannedValue = new TreeMap();   		   		 	
   		if (reportBeanWithHistoryList!=null) {
   			Calendar calendarStartDate = Calendar.getInstance();
   			Calendar calendarEndDate = Calendar.getInstance();
   			Calendar calendar = Calendar.getInstance();
   			//to avoid that the last days of the year to be taken as the first 
   			//week of the now year but the year remains the old one
   			calendar.setMinimalDaysInFirstWeek(7);
   			Iterator<ReportBeanWithHistory> iterator = reportBeanWithHistoryList.iterator();
   			int calendarInterval = DatasourceCommonBL.getCalendarInterval(timeInterval);   			
   			while (iterator.hasNext()) {
   				ReportBeanWithHistory reportBeanWithHistory = iterator.next();	
				TWorkItemBean workItemBean = reportBeanWithHistory.getWorkItemBean();
				Integer projectID = workItemBean.getProjectID();
				Date startDate = workItemBean.getStartDate();
				if (startDate==null) {
					startDate = minStartDate;
				}
				Date endDate = workItemBean.getEndDate();
				if (endDate==null) {
					endDate = maxEndDate;
				}				
				calendarStartDate.setTime(startDate);
				calendarEndDate.setTime(endDate);				
				List<TBudgetBean> budgetBeans = reportBeanWithHistory.getBudgetHistory();
				if (budgetBeans==null || budgetBeans.size()==0) {
					continue;
				} 
				TBudgetBean budgetBean = budgetBeans.get(0);
				Double effort;
				if (isTime) {
					effort = budgetBean.getEstimatedHours();
					Integer timeUnits = budgetBean.getTimeUnit();
					if (timeUnits!=null &&  AccountingBL.TIMEUNITS.WORKDAYS.equals(timeUnits.intValue())) {
						//transform to hours because the actual costs are always on hours
						effort = AccountingBL.transformToTimeUnits(effort, hoursPerWorkingDayMap.get(projectID), timeUnits, AccountingBL.TIMEUNITS.HOURS);
					}
				} else {
					effort = budgetBean.getEstimatedCost();
				}
				if (effort==null || effort.doubleValue()==0.0) {
					continue;
				}
				int noOfPeriods = DatasourceCommonBL.getNumberOfPeriods(calendarStartDate, calendarEndDate, timeInterval);
				Double deltaPerPeriod;
				if (noOfPeriods==0) {
					//probably startDate == endDate
					deltaPerPeriod = effort;
				} else {
					deltaPerPeriod = AccountingBL.roundToDecimalDigits(effort/noOfPeriods, isTime);
				}
				calendar.setTime(startDate);				
				int yearValue = calendarStartDate.get(Calendar.YEAR);
				int intervalValue = calendarStartDate.get(calendarInterval);
				boolean isFirst = true;									 
				while (calendar.before(calendarEndDate) || isFirst) {						
					if (isFirst) {
						isFirst = false;
					} else {
						calendar.add(calendarInterval, 1);
					}						
					yearValue = calendar.get(Calendar.YEAR);
					intervalValue = calendar.get(calendarInterval);
					
					
					
					SortedMap<Integer, Double> intervalToPlannedValues = yearToIntervalToPlannedValue.get(new Integer(yearValue));
					if (intervalToPlannedValues==null) {
						yearToIntervalToPlannedValue.put(new Integer(yearValue), new TreeMap<Integer, Double>());
						intervalToPlannedValues = (SortedMap)yearToIntervalToPlannedValue.get(new Integer(yearValue));
					}
					Double totalPlannedValueForInterval = (Double)intervalToPlannedValues.get(new Integer(intervalValue));
					if (totalPlannedValueForInterval==null) {
						totalPlannedValueForInterval = deltaPerPeriod;
					} else {
						totalPlannedValueForInterval = new Double(totalPlannedValueForInterval.doubleValue()+deltaPerPeriod.doubleValue());						
					}
					intervalToPlannedValues.put(new Integer(intervalValue), totalPlannedValueForInterval);					
				}											
			}
   		}        	
   		return yearToIntervalToPlannedValue;
   	}	
	
   	static SortedMap<Integer, SortedMap<Integer, Double>> calculateActualValues(
			List<ReportBeanWithHistory> reportBeanWithHistoryList, 
			boolean isTime, int timeInterval) {
		SortedMap<Integer, SortedMap<Integer, Double>> yearToIntervalToActualValue = new TreeMap();  	
		if (reportBeanWithHistoryList!=null) {
			Calendar calendar = Calendar.getInstance();
			//to avoid that the last days of the year to be taken as the first 
   			//week of the now year but the year remains the old one
			calendar.setMinimalDaysInFirstWeek(7);
			Iterator<ReportBeanWithHistory> iterator = reportBeanWithHistoryList.iterator();
			int calendarInterval = DatasourceCommonBL.getCalendarInterval(timeInterval);   			
			while (iterator.hasNext()) {
				ReportBeanWithHistory reportBeanWithHistory = iterator.next();								
				List<TCostBean> costBeans = reportBeanWithHistory.getCosts();
				if (costBeans==null || costBeans.size()==0) {
					continue;
				} 
				int yearValue;
				int intervalValue;
				Iterator<TCostBean> itrCostBeans = costBeans.iterator();
				while (itrCostBeans.hasNext()) {
					TCostBean costBean = (TCostBean) itrCostBeans.next();
					Double effortValue;
					if (isTime) {
						effortValue = costBean.getHours();
					} else {
						effortValue = costBean.getCost();
					}
					if (effortValue==null || effortValue.doubleValue()==0.0) {
						continue;
					}
					Date effortDate = costBean.getEffortdate();
					if (effortDate==null) {
						effortDate = costBean.getLastEdit();
					}
					if (effortDate!=null) {
						calendar.setTime(effortDate);
						yearValue = calendar.get(Calendar.YEAR);
						intervalValue = calendar.get(calendarInterval);						
						SortedMap<Integer, Double> intervalToActualValues = yearToIntervalToActualValue.get(new Integer(yearValue));
						if (intervalToActualValues==null) {
							yearToIntervalToActualValue.put(new Integer(yearValue), new TreeMap<Integer, Double>());
							intervalToActualValues = (SortedMap)yearToIntervalToActualValue.get(new Integer(yearValue));
						}
						Double totalActualEffortForInterval = (Double)intervalToActualValues.get(new Integer(intervalValue));
						if (totalActualEffortForInterval==null) {
							totalActualEffortForInterval = effortValue;
						} else {
							totalActualEffortForInterval = new Double(totalActualEffortForInterval.doubleValue()+effortValue.doubleValue());						
						}
						intervalToActualValues.put(new Integer(intervalValue), totalActualEffortForInterval);
					}					
				}
			}
		}        		
		return yearToIntervalToActualValue;
	}

	
	
	static SortedMap<Integer, SortedMap<Integer, Double>> calculateEarnedValues(List<ReportBeanWithHistory> reportBeanWithHistoryList, 
			boolean isTime, Date minStartDate, Date maxEndDate, Integer timeInterval, Map<Integer, Double> hoursPerWorkingDayMap, Set<Integer> finalStates) {
		SortedMap<Integer, SortedMap<Integer, Double>> yearToIntervalToPlannedValue = new TreeMap();   		   		 	
		if (reportBeanWithHistoryList!=null) {
			Calendar calendarStartDate = Calendar.getInstance();
			Calendar calendarEndDate = Calendar.getInstance();
			Calendar calendar = Calendar.getInstance();
			//to avoid that the last days of the year to be taken as the first 
   			//week of the now year but the year remains the old one
			calendar.setMinimalDaysInFirstWeek(7);
			Iterator<ReportBeanWithHistory> iterator = reportBeanWithHistoryList.iterator();
			int calendarInterval = DatasourceCommonBL.getCalendarInterval(timeInterval);   			
			while (iterator.hasNext()) {
				ReportBeanWithHistory reportBeanWithHistory = iterator.next();	
				TWorkItemBean workItemBean = reportBeanWithHistory.getWorkItemBean();
				Integer projectID = workItemBean.getProjectID();
				Date startDate = workItemBean.getStartDate();
				if (startDate==null) {
					startDate = minStartDate;
				}
				Date endDate = workItemBean.getEndDate();
				if (endDate==null) {
					endDate = maxEndDate;
				}				
				calendarStartDate.setTime(startDate);
				calendarEndDate.setTime(endDate);
				
				Map<Integer, Map<Integer, HistoryValues>> historyMap = reportBeanWithHistory.getHistoryValuesMap();
				List<HistoryValues> historyValuesList = HistoryLoaderBL.getHistoryValuesList(historyMap, false);
				if (historyValuesList==null || historyValuesList.size()==0) {
					continue;
				}
				//the first HistoryValue should be the most recent because of sorting in HistoryLoaderBL.getHistoryValuesList()
				HistoryValues historyValues = historyValuesList.get(0);
				if (!SystemFields.INTEGER_STATE.equals(historyValues.getFieldID())) {
					//should never happen
					continue;
				}
				Integer stateID = (Integer)historyValues.getNewValue();
				if (stateID==null || !finalStates.contains(stateID)) {
					continue;
				}
				Date lastStateChangeDate = historyValues.getLastEdit(); 
				
				//state is a final state get the earned value as the total effort
				List<TBudgetBean> budgetBeans = reportBeanWithHistory.getBudgetHistory();
				if (budgetBeans==null || budgetBeans.size()==0) {
					continue;
				} 
				TBudgetBean budgetBean = budgetBeans.get(0);
				Double effort;
				if (isTime) {
					effort = budgetBean.getEstimatedHours();
					Integer timeUnits = budgetBean.getTimeUnit();
					if (timeUnits!=null &&  AccountingBL.TIMEUNITS.WORKDAYS.equals(timeUnits.intValue())) {
						//transform to hours because the actual costs are always on hours
						effort = AccountingBL.transformToTimeUnits(effort, hoursPerWorkingDayMap.get(projectID), timeUnits, AccountingBL.TIMEUNITS.HOURS);
					}
				} else {
					effort = budgetBean.getEstimatedCost();
				}
				if (effort==null || effort.doubleValue()==0.0) {
					continue;
				}				 								
				calendar.setTime(lastStateChangeDate);				
				int yearValue = calendar.get(Calendar.YEAR);
				int intervalValue = calendar.get(calendarInterval);		
				SortedMap<Integer, Double> intervalToPlannedValues = yearToIntervalToPlannedValue.get(new Integer(yearValue));
				if (intervalToPlannedValues==null) {
					yearToIntervalToPlannedValue.put(new Integer(yearValue), new TreeMap<Integer, Double>());
					intervalToPlannedValues = (SortedMap)yearToIntervalToPlannedValue.get(new Integer(yearValue));
				}
				Double totalEarnedValueForInterval = (Double)intervalToPlannedValues.get(new Integer(intervalValue));
				if (totalEarnedValueForInterval==null) {
					totalEarnedValueForInterval = new Double(effort);
				} else {
					totalEarnedValueForInterval = new Double(totalEarnedValueForInterval.doubleValue()+effort);						
				}
				intervalToPlannedValues.put(new Integer(intervalValue), totalEarnedValueForInterval);															
			}
		}        	
		return yearToIntervalToPlannedValue;
	}
	
	static void accumulateValues(SortedMap<Integer, SortedMap<Integer, Double>> yearToIntervalToValues) {
		if (yearToIntervalToValues!=null) {
			Double acumulatedValue = new Double(0.0);
			Iterator<Integer> itrYear = yearToIntervalToValues.keySet().iterator();
			while (itrYear.hasNext()) {
				Integer year = (Integer) itrYear.next();
				SortedMap<Integer, Double> intervalToValues = yearToIntervalToValues.get(year);
				if (intervalToValues!=null) {
					Iterator<Integer> itrInterval = intervalToValues.keySet().iterator();
					while (itrInterval.hasNext()) {
						 Integer interval = itrInterval.next();
						 Double actualValue = intervalToValues.get(interval);
						 if (actualValue!=null) {
							 acumulatedValue = new Double(acumulatedValue.doubleValue() + actualValue.doubleValue());
						 }
						 intervalToValues.put(interval, acumulatedValue);
					}
				}
			}
		}
	}

	/**
	 * Add a value type to the EarnedValueTimeSlice map
	 * @param timeSliceMap
	 * @param valuesMap
	 * @param valueType
	 */
	static void addValueTypeToBeans(SortedMap<Date, EarnedValueTimeSlice> timeSliceMap, 
			SortedMap<Date, Object> valuesMap, int valueType) {		
		if (valuesMap!=null) {
			Iterator dateIterator = valuesMap.keySet().iterator();
			while (dateIterator.hasNext()) {
				Date date = (Date) dateIterator.next();
				EarnedValueTimeSlice earnedValueTimeSlice = (EarnedValueTimeSlice)timeSliceMap.get(date);
				if (earnedValueTimeSlice==null) {
					earnedValueTimeSlice = new EarnedValueTimeSlice();
					timeSliceMap.put(date, earnedValueTimeSlice);
				}
				earnedValueTimeSlice.setDate(date);
				switch (valueType) {
				case EarnedValueDatasource.PLANNED_VALUE:
					earnedValueTimeSlice.setPlannedValue((Double)valuesMap.get(date));
					break;
				case EarnedValueDatasource.ACTUAL_EFFORT:
					earnedValueTimeSlice.setActualCost((Double)valuesMap.get(date));
					break;
				case EarnedValueDatasource.EARNED_VALUE:
					earnedValueTimeSlice.setEarnedvalue((Double)valuesMap.get(date));
					break;
	
				}
			}
		}
	}
	
	static Document convertToDOM(Collection openClosedJavaBeans, Locale locale, String bundleName,
			String personName, int selectedTimeInterval, Integer effortType) {		
		Document dom = null;
		try{
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
			DocumentBuilder builder = factory.newDocumentBuilder ();
			dom = builder.newDocument ();	
		}catch (FactoryConfigurationError e){
			logger.error ("Creating the DOM document failed with FactoryConfigurationError:" + e.getMessage());
			return null;
		}catch (ParserConfigurationException e){
			logger.error ("Creating the DOM document failed with ParserConfigurationException: " + e.getMessage());
			return null;
		}
		Element root = dom.createElement ("track-report");
        Element createdBy =createDomElement("createdBy",personName,dom);
        root.appendChild(createdBy);        
        Element timeInterval =createDomElement("timeInterval", 
        		DatasourceCommonBL.getLocalizedTimeInterval(locale, bundleName, selectedTimeInterval), dom);
        root.appendChild(timeInterval);
        Element effortTypeElement =createDomElement("effortType", 
        		DatasourceCommonBL.getLocalizedEffortType(locale, bundleName, effortType), dom);
        root.appendChild(effortTypeElement); 
        Iterator iterator = openClosedJavaBeans.iterator();
        while (iterator.hasNext()) {
        	EarnedValueTimeSlice earnedValueTimeSlice = (EarnedValueTimeSlice) iterator.next();									
            Element period = dom.createElement ("period");            
          	period.appendChild(createDomElement("date", DateTimeUtils.formatISODateTime(earnedValueTimeSlice.getDate()), dom));
        	boolean isTime = false;
        	if (effortType==DatasourceCommonBL.EFFORT_TYPE.TIME) {
        		isTime = true;
        	}
          	String plannedValue = null;
          	if (earnedValueTimeSlice.getPlannedValue()!=null) {
          		plannedValue = AccountingBL.getFormattedDouble(earnedValueTimeSlice.getPlannedValue(), Locale.ENGLISH, isTime);
          	}
          	period.appendChild(createDomElement("plannedValue", plannedValue, dom));
          	
          	String actualValue = null;
          	if (earnedValueTimeSlice.getActualCost()!=null) {
          		actualValue = AccountingBL.getFormattedDouble(earnedValueTimeSlice.getActualCost(), Locale.ENGLISH, isTime);           			
          	}
          	period.appendChild(createDomElement("actualValue", actualValue, dom));
			
          	String earnedValue = null;
          	if (earnedValueTimeSlice.getEarnedvalue()!=null) {
          		earnedValue = AccountingBL.getFormattedDouble(earnedValueTimeSlice.getEarnedvalue(), Locale.ENGLISH, isTime);          			
          	}
          	period.appendChild(createDomElement("earnedValue", earnedValue, dom));          	
          	root.appendChild (period);			
		}
		dom.appendChild (root);
        return dom;
	}
	
	/**
	 * @param elementName
	 * @param elementValue
	 * @param dom
	 * @return
	 */
	private static Element createDomElement (String elementName, String elementValue, Document dom) {
		Element element = dom.createElement (elementName);
		if (elementValue == null || "".equals(elementValue.trim())) {
			element.appendChild (dom.createTextNode (""));
		} else {
			element.appendChild (dom.createTextNode(elementValue));
		}						
		return element;
	}
}
