package com.trackplus.plugin.report.earnedValue;

import java.io.OutputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;

import com.aurel.track.beans.TPersonBean;
import com.aurel.track.fieldType.constants.SystemFields;
import com.aurel.track.item.history.HistoryLoaderBL.LONG_TEXT_TYPE;
import com.aurel.track.persist.ReportBeanHistoryLoader;
import com.aurel.track.plugin.DatasourceDescriptor;
import com.aurel.track.report.datasource.DatasourceCommonBL;
import com.aurel.track.report.datasource.IPluggableDatasource;
import com.aurel.track.report.execute.ReportBeanWithHistory;
import com.aurel.track.report.export.ReportBeansToXML;
import com.aurel.track.util.GeneralUtils;
import com.aurel.track.util.StringArrayParameterUtils;

public class EarnedValueDatasource implements IPluggableDatasource {
	private static Logger logger = Logger.getLogger(EarnedValueDatasource.class);
	
	static final int PLANNED_VALUE = 1;
	static final int ACTUAL_EFFORT = 2;
	static final int EARNED_VALUE = 3;
		
	
	//Configuration page constants
	protected static interface PARAMETER_NAME {	   
	    static String EFFORTTYPES = "effortTypes";
	    static String SELECTED_EFFORTTYPE = "selectedEffortType";
	    static String STATUSES = "statuses";
		static String SELECTED_FINALSTATUSES = "selectedFinalStatuses";		
	}
	
	/**
	 * Gets the data source object retrieved using the parameters settings
	 * @param parameters
	 * @param datasourceDescriptor
	 * @param personID the logged in person
	 * @param locale
	 * @return
	 */
	public Object getDatasource(Map parameters, DatasourceDescriptor datasourceDescriptor, 
			TPersonBean personBean, Locale locale) {		    
    	//get the workItemBeans
    	List reportBeanList = DatasourceCommonBL.getReportBeanList(parameters, personBean, locale);    	
    	if (reportBeanList==null || reportBeanList.size()==0) {
    		return null;
    	}
    	logger.debug("Total number of reportBeans by datasource " + reportBeanList.size());
    	Date dateFrom = DatasourceCommonBL.getDateFrom(parameters, locale);
    	Date dateTo = DatasourceCommonBL.getDateTo(parameters, locale);    	
    	List<ReportBeanWithHistory> reportBeanWithHistoryList = 
    		ReportBeanHistoryLoader.getReportBeanWithHistoryList(
				reportBeanList, locale,	false, false, true, 
				new Integer[]{SystemFields.INTEGER_STATE}, true, true, true, false, null, false,
				dateFrom, dateTo, true, LONG_TEXT_TYPE.ISPLAIN);    	
    	//get the selected time interval
    	int effortType = StringArrayParameterUtils.parseIntegerValue(parameters, 
				PARAMETER_NAME.SELECTED_EFFORTTYPE, DatasourceCommonBL.EFFORT_TYPE.TIME);    	
    	boolean isTime = false;
    	if (effortType==DatasourceCommonBL.EFFORT_TYPE.TIME) {
    		isTime = true;
    	}
    	Set<Integer> projectIDs = EarnedValueBL.prefilterReportBeansGetProjectIDs(reportBeanWithHistoryList, isTime, dateFrom, dateTo);
    	SortedMap<Date, EarnedValueTimeSlice> earnedValueTimeSliceMap = new TreeMap();
    	int selectedTimeInterval = StringArrayParameterUtils.parseIntegerValue(parameters, 
    			DatasourceCommonBL.PARAMETER_NAME.SELECTED_TIME_INTERVAL, DatasourceCommonBL.TIME_INTERVAL.MONTH);    	
    	if (reportBeanWithHistoryList.size()>0) {
    		logger.debug("Number of reportBeanWithHistoryList after removing items with no budget or overlapping start/end dates " + reportBeanWithHistoryList.size());    		
    		Date minStartDate = EarnedValueBL.getMinMaxStartEndDate(reportBeanWithHistoryList, true);    		
    		Date maxEndDate = EarnedValueBL.getMinMaxStartEndDate(reportBeanWithHistoryList, false);
    		if (minStartDate!=null && maxEndDate!=null) {
    			//at least one startdate and enddate is specified
		    	Map<Integer, Double> hoursPerWorkingDayMap = EarnedValueBL.getHoursPerWorkingDayMap(projectIDs);
		    	    	
		    	SortedMap<Integer, SortedMap<Integer, Double>> plannedValues = EarnedValueBL.calculatePlannedValues(
		    			reportBeanWithHistoryList, isTime, minStartDate, maxEndDate, selectedTimeInterval, hoursPerWorkingDayMap);
		    	SortedMap<Integer, SortedMap<Integer, Double>> actualValues =  EarnedValueBL.calculateActualValues(
		    			reportBeanWithHistoryList, isTime, selectedTimeInterval);
		    	Integer[] selectedStatuses = StringArrayParameterUtils.parseIntegerArrValue(parameters, PARAMETER_NAME.SELECTED_FINALSTATUSES);
		    	SortedMap<Integer, SortedMap<Integer, Double>> earnedValues = EarnedValueBL.calculateEarnedValues(
		    			reportBeanWithHistoryList, isTime, minStartDate, maxEndDate, 
						selectedTimeInterval, hoursPerWorkingDayMap, GeneralUtils.createSetFromIntegerArr(selectedStatuses));    	
		    	DatasourceCommonBL.addZerosForEmptyIntervals(dateFrom, dateTo, selectedTimeInterval, plannedValues, false);
		    	DatasourceCommonBL.addZerosForEmptyIntervals(dateFrom, dateTo, selectedTimeInterval, actualValues, false);
		    	DatasourceCommonBL.addZerosForEmptyIntervals(dateFrom, dateTo, selectedTimeInterval, earnedValues, false);    	
		    	EarnedValueBL.accumulateValues(plannedValues);
		    	EarnedValueBL.accumulateValues(actualValues);
		    	EarnedValueBL.accumulateValues(earnedValues);    	
		    	SortedMap<Date, Object> plannedDateValues = DatasourceCommonBL.transformPeriodsToDates(plannedValues, selectedTimeInterval);
		    	SortedMap<Date, Object> plannedActualValues = DatasourceCommonBL.transformPeriodsToDates(actualValues, selectedTimeInterval);
		    	SortedMap<Date, Object> plannedEarnedValues = DatasourceCommonBL.transformPeriodsToDates(earnedValues, selectedTimeInterval);    	
		    	EarnedValueBL.addValueTypeToBeans(earnedValueTimeSliceMap, 
		    			plannedDateValues, PLANNED_VALUE);
		    	EarnedValueBL.addValueTypeToBeans(earnedValueTimeSliceMap, 
		    			plannedActualValues, ACTUAL_EFFORT);    	
		    	EarnedValueBL.addValueTypeToBeans(earnedValueTimeSliceMap, 
		    			plannedEarnedValues, EARNED_VALUE);
    		}
    	}
    	return EarnedValueBL.convertToDOM(earnedValueTimeSliceMap.values(), locale, 
    			datasourceDescriptor.getBundleName(), personBean.getFullName(), selectedTimeInterval, effortType);    		   
	}
	
	/**
	 * Serializes the datasource in an XML file
	 */
	public void serializeDatasource(OutputStream outputStream,
			Object datasource) {
		ReportBeansToXML.convertToXml(outputStream, (Document)datasource);
	}

	/**
	 * Implements the serialization
	 */
	public boolean implementSerialization() {
		return true;
	}
			
    /**
	 * Prepares a map for rendering
	 * @param parameters
	 * @param datasourceDescriptor
	 * @param personID
	 * @param locale 
	 * @return
	 */
	public Map prepareParameters(Map parameters, DatasourceDescriptor datasourceDescriptor, 
			TPersonBean personBean, Locale locale) {
		String bundleName = datasourceDescriptor.getBundleName();
		if (parameters==null) {
			parameters = new HashMap();
		}
		DatasourceCommonBL.prepareCommonChartParameters(parameters, datasourceDescriptor, personBean, locale);
		
		/*parameters.put(DatasourceCommonBL.PARAMETER_NAME.locale, locale);
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.bundleName, bundleName);		
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.jsCalDateFormat, DateTimeUtils.getInstance().getJsCalDateFormat(locale));
		
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.DATASOURCE_TYPES, DatasourceCommonBL.getDatasourceTypes(locale, bundleName));
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_DATASOURCE_TYPE, 
	    		new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, 
	    				DatasourceCommonBL.PARAMETER_NAME.SELECTED_DATASOURCE_TYPE, DatasourceCommonBL.DATASOURCE_TYPE.PROJECT_RELEASE)));
		
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.PROJECTS, DatasourceCommonBL.getProjectsAndReleases(personBean.getObjectID()));
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_PROJECT, StringArrayParameterUtils.parseIntegerValue(parameters, 
				DatasourceCommonBL.PARAMETER_NAME.SELECTED_PROJECT));
		
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.QUERYTYPES, DatasourceCommonBL.getQueryTypeList(locale));
		Integer selectedQueryType = new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, 
        		DatasourceCommonBL.PARAMETER_NAME.SELECTED_QUERYTYPE, TQueryRepositoryBean.REPORT_QUERY_TYPE.TREE_FILTER));
		//set back in parameters as Integer instead of the original String[] because otherwise the selection is lost
        //the listKey type of the list elements (Integer) would differ from the type of the selected value (String[])
        parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_QUERYTYPE, selectedQueryType);

    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.REPOSITORYTYPES, DatasourceCommonBL.getRepositoryTypeList(locale));
    	Integer selectedRepositoryType = new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, 
    			DatasourceCommonBL.PARAMETER_NAME.SELECTED_REPOSITORYTYPE, TQueryRepositoryBean.REPOSITORY_TYPE.PRIVATE)); 
    	//set back in parameters as Integer instead of the original String[] because otherwise the selection is lost
        //the listKey type of the list elements (Integer) would differ from the type of the selected value (String[])
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_REPOSITORYTYPE, selectedRepositoryType);
    	
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.QUERIES, 
    			QueryBL.loadReportQueryBeans(personBean.getObjectID(), null, 
    					new Integer[] {selectedQueryType}, new Integer[]{selectedRepositoryType}));
    	    	    	    	
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.PERIOD_TYES, DatasourceCommonBL.possiblePeriodTypes(locale, bundleName));
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_PERIOD_TYPE, 
	    		new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, 
	    				DatasourceCommonBL.PARAMETER_NAME.SELECTED_PERIOD_TYPE, DatasourceCommonBL.PERIOD_TYPE.FROM_TO)));
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.DATE_FROM,
	    		StringArrayParameterUtils.getStringValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DATE_FROM));
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.DATE_TO, 
    			StringArrayParameterUtils.getStringValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DATE_TO));
    	parameters.put(DatasourceCommonBL.PERIOD_TYPE.DAYS_BEFORE, 
	    		new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DAYS_BEFORE, 0)));
    	
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.TIME_INTERVALS, DatasourceCommonBL.possibleTimeIntervals(locale, bundleName));
    	int selectedTimeInterval = StringArrayParameterUtils.parseIntegerValue(parameters, 
    			DatasourceCommonBL.PARAMETER_NAME.SELECTED_TIME_INTERVAL, DatasourceCommonBL.TIME_INTERVAL.MONTH);   
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_TIME_INTERVAL, new Integer(selectedTimeInterval));
    	*/
    	parameters.put(PARAMETER_NAME.EFFORTTYPES, DatasourceCommonBL.possibleEffortTypes(locale, bundleName));    	
    	int effortType = StringArrayParameterUtils.parseIntegerValue(parameters, 
				PARAMETER_NAME.SELECTED_EFFORTTYPE, DatasourceCommonBL.EFFORT_TYPE.TIME);
    	parameters.put(PARAMETER_NAME.SELECTED_EFFORTTYPE, new Integer(effortType));
    	
    	parameters.put(EarnedValueDatasource.PARAMETER_NAME.STATUSES, EarnedValueBL.getStatuses(locale));
    	Integer[] selectedStatuses = StringArrayParameterUtils.parseIntegerArrValue(parameters, PARAMETER_NAME.SELECTED_FINALSTATUSES);
    	if (selectedStatuses==null || selectedStatuses.length==0) {
    		selectedStatuses = GeneralUtils.getBeanIDs(EarnedValueBL.loadClosedStates());
    	}
	    parameters.put(PARAMETER_NAME.SELECTED_FINALSTATUSES, selectedStatuses);	    
	    return parameters;
		
	}
    
	/**
	 * Get the names of those parameter names which should be saved in the session
	 * for future use of the same report configuration
	 * @return
	 */
	public List getSavedParameterNames() {
		List savedParameterNames = DatasourceCommonBL.getCommonSavedParameterNames();
		savedParameterNames.add(EarnedValueDatasource.PARAMETER_NAME.SELECTED_FINALSTATUSES);		
		return savedParameterNames;
	}
 	
}
