package com.trackplus.plugin.report.filterHistory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import com.aurel.track.beans.TFieldChangeBean;
import com.aurel.track.beans.TFieldConfigBean;
import com.aurel.track.beans.TPersonBean;
import com.aurel.track.beans.TReleaseBean;
import com.aurel.track.dao.DAOFactory;
import com.aurel.track.dao.FieldConfigDAO;
import com.aurel.track.dao.PersonDAO;
import com.aurel.track.dao.ProjectDAO;
import com.aurel.track.dao.ReleaseDAO;
import com.aurel.track.fieldType.constants.SystemFields;
import com.aurel.track.item.history.HistoryLoaderBL;
import com.aurel.track.item.history.HistoryLoaderBL.LONG_TEXT_TYPE;
import com.aurel.track.persist.ReportBeanHistoryLoader;
import com.aurel.track.report.execute.ReportBean;
import com.aurel.track.report.execute.ReportBeanWithHistory;
import com.aurel.track.resources.LocalizeUtil;
import com.aurel.track.util.GeneralUtils;
import com.aurel.track.util.IntegerStringBean;

public class FilterHistoryBL {
	
	private static ProjectDAO projectDAO = DAOFactory.getFactory().getProjectDAO();    
	private static ReleaseDAO releaseDAO = DAOFactory.getFactory().getReleaseDAO();
	private static FieldConfigDAO fieldConfigDAO = DAOFactory.getFactory().getFieldConfigDAO();
	private static PersonDAO personDAO = DAOFactory.getFactory().getPersonDAO();
	
	public static List<ReportBeanWithHistory> addHistoryData(FilterHistoryConfig filterHistoryConfig, Locale locale) {
		List<ReportBean> reportBeanList = filterHistoryConfig.getReportBeanList();
		Boolean excludeWithoutHistory = filterHistoryConfig.getExcludeWithoutHistory();
		Date fromDate = filterHistoryConfig.getDateFrom();
		Date toDate = filterHistoryConfig.getDateTo();
		Integer[] fields = filterHistoryConfig.getFields();
		Integer personID = filterHistoryConfig.getPerson();
		Set fieldsSet = GeneralUtils.createSetFromIntArr(GeneralUtils.createIntArrFromIntegerArr(fields));
		
		boolean includeComments = false;
		boolean includeBudget = false;
		boolean includeCost = false;
		boolean includeFieldChange = false;
		Integer[] systemOrCustomFieldIDs = null;
		if (fieldsSet.isEmpty()) {
			//no field selected means all fields
			includeComments = true;
			includeBudget = true;
			includeCost = true;
			includeFieldChange = true;
		} else {
			//at least one field is selected
			if (fieldsSet.contains(SystemFields.INTEGER_COMMENT)) {
				fieldsSet.remove(SystemFields.INTEGER_COMMENT);
				includeComments = true;
			}
			if (fieldsSet.contains(SystemFields.INTEGER_BUDGET_HISTORY)) {
				fieldsSet.remove(SystemFields.INTEGER_BUDGET_HISTORY);
				includeBudget = true;
			}
			if (fieldsSet.contains(SystemFields.INTEGER_COST_HISTORY)) {
				fieldsSet.remove(SystemFields.INTEGER_COST_HISTORY);
				includeCost = true;
			}						
			if (fieldsSet.size()>0) {
				systemOrCustomFieldIDs = GeneralUtils.createIntegerArrFromSet(fieldsSet);
				includeFieldChange = true;
			}
		}		
		List<ReportBeanWithHistory> reportBeanWithHistoryList = ReportBeanHistoryLoader.getReportBeanWithHistoryList(
				reportBeanList, locale,	false, includeComments, includeFieldChange, 
				systemOrCustomFieldIDs, includeBudget, includeCost, true, false, personID, true,
				fromDate, toDate, true, LONG_TEXT_TYPE.ISPLAIN);
		if (excludeWithoutHistory!=null && excludeWithoutHistory.booleanValue()) {
			//filter out those values which does not have any filtered history entry
			if (reportBeanWithHistoryList!=null && reportBeanWithHistoryList.size()>0) {
				Iterator<ReportBeanWithHistory> iterator = reportBeanWithHistoryList.iterator();
				while (iterator.hasNext()) {
					ReportBeanWithHistory reportBeanWithHistory = (ReportBeanWithHistory) iterator.next();
					if (!hasHistory(reportBeanWithHistory)) {
						iterator.remove();
					}
				}
			}
		} 
		return reportBeanWithHistoryList;
		
	}
	
	public static boolean hasHistory(ReportBeanWithHistory reportBeanWithHistory) {
		if (reportBeanWithHistory.getHistoryValuesMap()!=null && reportBeanWithHistory.getHistoryValuesMap().size()>0) {
			return true;
		}
		if (reportBeanWithHistory.getComments()!=null && reportBeanWithHistory.getComments().size()>0) {
			return true;
		}
		if (reportBeanWithHistory.getBudgetHistory()!=null && reportBeanWithHistory.getBudgetHistory().size()>0) {
			return true;
		}
		if (reportBeanWithHistory.getCosts()!=null && reportBeanWithHistory.getCosts().size()>0) {
			return true;
		}
		return false;
	}
	
	/**
	 * Get the list of possible fields
	 * @param locale
	 * @return
	 */
	public static List<IntegerStringBean> getFields(Locale locale) {
		List<IntegerStringBean> fieldsList = new ArrayList<IntegerStringBean>();		
		Set<Integer> pseudoHistoryFields = HistoryLoaderBL.getPseudoHistoryFields();
		Iterator<Integer> itrPseudoHistoryFields = pseudoHistoryFields.iterator();
		while (itrPseudoHistoryFields.hasNext()) {
			Integer pseudoHistoryField = (Integer) itrPseudoHistoryFields.next();
			String resourceKey = HistoryLoaderBL.getHistoryFieldKey(pseudoHistoryField);
			fieldsList.add(new IntegerStringBean(LocalizeUtil.getLocalizedTextFromApplicationResources(
					resourceKey, locale), pseudoHistoryField));
		}
		/*fieldsList.add(new IntegerStringBean(LocalizeUtil.getLocalizedTextFromApplicationResources(
				HistoryLoaderBL.COMMON_FIELD_KEY, locale), TFieldChangeBean.COMPOUND_HISTORY_FIELD));*/
		fieldsList.add(new IntegerStringBean(LocalizeUtil.getLocalizedTextFromApplicationResources(
				HistoryLoaderBL.BUDGET, locale), SystemFields.INTEGER_BUDGET_HISTORY));
		fieldsList.add(new IntegerStringBean(LocalizeUtil.getLocalizedTextFromApplicationResources(
				HistoryLoaderBL.COST, locale), SystemFields.INTEGER_COST_HISTORY));
		
		Set<Integer> notRelevantHistoryFields = getNotRelevantHistoryFields(); 
		List<TFieldConfigBean> allDefaultConfigs=LocalizeUtil.localizeFieldConfigs(fieldConfigDAO.loadDefault(), locale);
		Iterator<TFieldConfigBean> itrFieldConfigs = allDefaultConfigs.iterator();
		while (itrFieldConfigs.hasNext()) {
			TFieldConfigBean fieldConfigBean = (TFieldConfigBean) itrFieldConfigs.next();
			if (!notRelevantHistoryFields.contains(fieldConfigBean.getField())) {
				fieldsList.add(new IntegerStringBean(fieldConfigBean.getLabel(), fieldConfigBean.getField()));
			}
		}
		Collections.sort(fieldsList);
		return fieldsList;
	}
	
	private static Set<Integer> getNotRelevantHistoryFields() {
		Set<Integer> notRelevantFields = new HashSet<Integer>();
		notRelevantFields.add(SystemFields.INTEGER_CHANGEDBY);
		notRelevantFields.add(SystemFields.INTEGER_ORIGINATOR);
		notRelevantFields.add(SystemFields.INTEGER_CREATEDATE);
		notRelevantFields.add(SystemFields.INTEGER_LASTMODIFIEDDATE);
		notRelevantFields.add(SystemFields.INTEGER_ISSUENO);
		return notRelevantFields;
	}
	
	/*private static Set<Integer> getAttachmentHistoryFields() {		
		Set<Integer> attachmentHistoryFields = new HashSet();
		attachmentHistoryFields.add(SystemFields.INTEGER_ATTACHMENT_ADD_HISTORY_FIELD);
		attachmentHistoryFields.add(SystemFields.INTEGER_ATTACHMENT_MODIFY_HISTORY_FIELD);
		attachmentHistoryFields.add(SystemFields.INTEGER_ATTACHMENT_DELETE_HISTORY_FIELD);		
		return attachmentHistoryFields;
	}*/
	
	/**
	 * Get the list of possible persons
	 * @param locale
	 * @return
	 */
	/*public static List<TPersonBean> getPersons(int[] workItemIDs, 
    		Integer[] fieldIDs, boolean includeField, Date fromDate, Date toDate) {
		return personDAO.loadHistoryChangedByPersons(workItemIDs, fieldIDs, includeField, fromDate, toDate);		
	}*/
	
	/**
	 * Get the list of possible persons
	 * @param locale
	 * @return
	 */
	public static List<TPersonBean> getPersons() {
		return personDAO.loadPersons1();		
	}
	
	/**
     * Validate the selected project/release 
     * @param selectedProjectsIDsAndReleaseIDs
     * @param allowedProjectsMap only those projects/releases should 
     * 	be added to the list for which the read right is not revoked
     * @param projectIDs
     * @param releaseIDs
     */
    public static Integer validateSelectedProjectOrReleaseID(Integer selectedProjectOrReleaseID, Integer personID) {    	
    	if (selectedProjectOrReleaseID!=null) {
    		Map allowedProjectsMap = GeneralUtils.createMapFromList(projectDAO.loadProjectsWithReadIssueRight(personID));    	    	
    		if (selectedProjectOrReleaseID.intValue()<0) {
    			if (allowedProjectsMap.containsKey(new Integer(-selectedProjectOrReleaseID.intValue()))) {
    				return selectedProjectOrReleaseID;
    			}
    		} else {
    			TReleaseBean releaseBean = releaseDAO.loadByPrimaryKey1(selectedProjectOrReleaseID);
    			if (releaseBean!=null && allowedProjectsMap.containsKey(releaseBean.getProjectID())) {
    				return selectedProjectOrReleaseID;
    			}
    		}
    	}
    	return null;    	        	
    }
	
}
