/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2009 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: FilterHistoryDatasource.java 2688 2010-11-04 16:47:10Z friedj $ 
 * 
 */

package com.trackplus.plugin.report.filterHistory;		
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.w3c.dom.Document;

import com.aurel.track.beans.TPersonBean;
import com.aurel.track.beans.TQueryRepositoryBean;
import com.aurel.track.plugin.DatasourceDescriptor;
import com.aurel.track.query.bl.QueryBL;
import com.aurel.track.report.datasource.DatasourceCommonBL;
import com.aurel.track.report.datasource.IPluggableDatasource;
import com.aurel.track.report.execute.ReportBeanWithHistory;
import com.aurel.track.report.export.ReportBeansToXML;
import com.aurel.track.util.DateTimeUtils;
import com.aurel.track.util.StringArrayParameterUtils;

public class FilterHistoryDatasource implements IPluggableDatasource {
		
	public static interface PARAMETER_NAME {		
		public static String EXCLUDE_WITHOUT_HISTORY = "excludeWithoutHistory";
		public static String FIELDS = "fields";
		public static String SELECTED_FIELDS = "selectedFields";		
		public static String PERSONS = "persons";
		public static String SELECTED_PERSON = "selectedPerson";	    	    
	    	
	}
	
	public Object getDatasource(Map parameters,
			DatasourceDescriptor datasourceDescriptor, TPersonBean personBean,
			Locale locale) {
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.locale, locale);
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.bundleName, datasourceDescriptor.getBundleName());
		FilterHistoryConfig filterHistoryConfig = getConfigData(parameters, personBean, locale);				
		List<ReportBeanWithHistory> historyData =  FilterHistoryBL.addHistoryData(filterHistoryConfig, locale);
		ReportBeansToXML reportBeansToXML = new ReportBeansToXML();
		return reportBeansToXML.convertToDOM(historyData, true, locale, personBean.getFullName(), null, null);
	}

	public void serializeDatasource(OutputStream outputStream,
			Object datasource) {
		ReportBeansToXML.convertToXml(outputStream, (Document)datasource);
	}

	public boolean implementSerialization() {
		return true;
	}
	
	
	
	/**
	 * Prepares the parameters map:
	 * 1. 	for first rendering
	 * 2. 	after each submit which needs re-rendering of the configuration page 
	 * 		(for example refreshing some lists etc., typically using a  
	 * 		 hidden field set through javascript before submit) 
	 * 3. 	after the submit for getting the datasource or generating the report     
	 */
	public Map prepareParameters(Map parameters, DatasourceDescriptor datasourceDescriptor, 
			TPersonBean personBean, Locale locale) {
		String bundleName = datasourceDescriptor.getBundleName();
		if (parameters == null) {
			//when no submit took place parameters is null 
			parameters = new HashMap();
		}
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.locale, locale);
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.bundleName, bundleName);
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.jsCalDateFormat, DateTimeUtils.getInstance().getJsCalDateFormat(locale));
			
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.DATASOURCE_TYPES, DatasourceCommonBL.getDatasourceTypes(locale, bundleName));	
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_DATASOURCE_TYPE, 
	    		new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, 
	    				DatasourceCommonBL.PARAMETER_NAME.SELECTED_DATASOURCE_TYPE, DatasourceCommonBL.DATASOURCE_TYPE.PROJECT_RELEASE)));
    	
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.PROJECTS, DatasourceCommonBL.getProjectsAndReleases(personBean.getObjectID()));
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_PROJECT, StringArrayParameterUtils.parseIntegerValue(parameters, 
				DatasourceCommonBL.PARAMETER_NAME.SELECTED_PROJECT));
		
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.QUERYTYPES, DatasourceCommonBL.getQueryTypeList(locale));
		Integer selectedQueryType = new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, 
        		DatasourceCommonBL.PARAMETER_NAME.SELECTED_QUERYTYPE, TQueryRepositoryBean.REPORT_QUERY_TYPE.TREE_FILTER));
        //set back in parameters as Integer instead of the original String[] because otherwise the selection is lost
        //the listKey type of the list elements (Integer) would differ from the type of the selected value (String[])
        parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_QUERYTYPE, selectedQueryType);
        
		parameters.put(DatasourceCommonBL.PARAMETER_NAME.REPOSITORYTYPES, DatasourceCommonBL.getRepositoryTypeList(locale));      	
    	Integer selectedRepositoryType = new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, 
    			DatasourceCommonBL.PARAMETER_NAME.SELECTED_REPOSITORYTYPE, TQueryRepositoryBean.REPOSITORY_TYPE.PRIVATE));    	
        //set back in parameters as Integer instead of the original String[] because otherwise the selection is lost
        //the listKey type of the list elements (Integer) would differ from the type of the selected value (String[])
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_REPOSITORYTYPE, selectedRepositoryType);

    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.QUERIES, QueryBL.loadReportQueryBeans(personBean.getObjectID(), null, 
				new Integer[] {selectedQueryType}, new Integer[]{selectedRepositoryType}));    	   
    	
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.PERIOD_TYES, DatasourceCommonBL.possiblePeriodTypes(locale, bundleName));
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.SELECTED_PERIOD_TYPE, 
	    		new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, 
	    				DatasourceCommonBL.PARAMETER_NAME.SELECTED_PERIOD_TYPE, DatasourceCommonBL.PERIOD_TYPE.FROM_TO)));    	
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.DATE_FROM, 
	    		StringArrayParameterUtils.getStringValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DATE_FROM));
    	parameters.put(DatasourceCommonBL.PARAMETER_NAME.DATE_TO, 
    			StringArrayParameterUtils.getStringValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DATE_TO));
    	parameters.put(DatasourceCommonBL.PERIOD_TYPE.DAYS_BEFORE, 
	    		new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DAYS_BEFORE, 0)));    	
    	parameters.put(PARAMETER_NAME.FIELDS, FilterHistoryBL.getFields(locale));
    	//set back in parameters as Integer[] instead of the original String[] because otherwise the selection is lost
        //the listKey type of the list elements (Integer) would differ from the type of the selected value (String[])
    	parameters.put(PARAMETER_NAME.SELECTED_FIELDS, StringArrayParameterUtils.parseIntegerArrValue(
    			parameters, PARAMETER_NAME.SELECTED_FIELDS));    	
    	parameters.put(PARAMETER_NAME.PERSONS, FilterHistoryBL.getPersons());
    	//set back in parameters as Integer instead of the original String[] because otherwise the selection is lost
        //the listKey type of the list elements (Integer) would differ from the type of the selected value (String[])
    	parameters.put(PARAMETER_NAME.SELECTED_PERSON, StringArrayParameterUtils.parseIntegerValue(parameters, 
    			PARAMETER_NAME.SELECTED_PERSON));
		return parameters;
	}
	
	/**
	 * Get the names of those parameter names which should be saved in the session
	 * for future use of the same report configuration
	 * @return
	 */
	public List getSavedParameterNames() {
		List savedParameterNames = new ArrayList();
		savedParameterNames.add(DatasourceCommonBL.PARAMETER_NAME.SELECTED_DATASOURCE_TYPE);
		savedParameterNames.add(DatasourceCommonBL.PARAMETER_NAME.SELECTED_PROJECT);
		savedParameterNames.add(DatasourceCommonBL.PARAMETER_NAME.SELECTED_QUERYTYPE);		
		savedParameterNames.add(DatasourceCommonBL.PARAMETER_NAME.SELECTED_REPOSITORYTYPE);
		savedParameterNames.add(DatasourceCommonBL.PARAMETER_NAME.SELECTED_QUERY);
		savedParameterNames.add(PARAMETER_NAME.EXCLUDE_WITHOUT_HISTORY);
		savedParameterNames.add(DatasourceCommonBL.PARAMETER_NAME.SELECTED_PERIOD_TYPE);
		savedParameterNames.add(DatasourceCommonBL.PARAMETER_NAME.DATE_FROM);
		savedParameterNames.add(DatasourceCommonBL.PARAMETER_NAME.DATE_TO);
		savedParameterNames.add(DatasourceCommonBL.PARAMETER_NAME.DAYS_BEFORE);
		savedParameterNames.add(PARAMETER_NAME.SELECTED_FIELDS);
		savedParameterNames.add(PARAMETER_NAME.SELECTED_PERSON);
		return savedParameterNames;
	}

	/**
	 * Get 	
	 * @param parameters
	 * @param userID
	 * @return
	 */
	protected FilterHistoryConfig getConfigData(Map parameters, TPersonBean personBean, Locale locale) {
		//loadParametersBeforeExecute(parameters, userID);		
		FilterHistoryConfig filterHistoryConfig = new FilterHistoryConfig();
		filterHistoryConfig.setReportBeanList(DatasourceCommonBL.getReportBeanList(parameters, personBean, locale));
		/*int dataSourceType = StringArrayParameterUtils.parseIntegerValue(parameters, 
				DatasourceCommonBL.PARAMETER_NAME.SELECTED_DATASOURCE_TYPE, DatasourceCommonBL.DATASOURCE_TYPE.PROJECT_RELEASE);						
		if (DatasourceCommonBL.DATASOURCE_TYPE.PROJECT_RELEASE==dataSourceType) {
			Integer projectOrRelease = new Integer(StringArrayParameterUtils.parseIntegerValue(parameters, 
					DatasourceCommonBL.PARAMETER_NAME.SELECTED_PROJECT, 0));			
			if (projectOrRelease!=null && projectOrRelease.intValue()!=0) {
	    		if (projectOrRelease.intValue()<0) {
	    			//project workItems
	    			filterHistoryConfig.setReportBeanList(ReportBeanLoader.getProjectReportBeans(new Integer(-projectOrRelease.intValue()), personBean.getObjectID(), locale));
	    		} else {
	    			//release workItems
	    			filterHistoryConfig.setReportBeanList(ReportBeanLoader.getReleaseReportBeans(projectOrRelease, personBean.getObjectID(), locale));
	    		}
			}			
		} else {
			int queryType = StringArrayParameterUtils.parseIntegerValue(parameters, 
					DatasourceCommonBL.PARAMETER_NAME.SELECTED_QUERYTYPE, TQueryRepositoryBean.REPORT_QUERY_TYPE.TREE_FILTER);		
			int repositoryType = StringArrayParameterUtils.parseIntegerValue(parameters, 
					DatasourceCommonBL.PARAMETER_NAME.SELECTED_REPOSITORYTYPE, TQueryRepositoryBean.REPOSITORY_TYPE.PRIVATE);
			Integer queryID = StringArrayParameterUtils.parseIntegerValue(parameters, DatasourceCommonBL.PARAMETER_NAME.SELECTED_QUERY);
			filterHistoryConfig.setReportBeanList(ReportBL.getReportBeansForQuery(queryID, 
					queryType, repositoryType, personBean, locale));			
		}*/
		
		Boolean excludeWithoutHistory = StringArrayParameterUtils.getBooleanValue(parameters, PARAMETER_NAME.EXCLUDE_WITHOUT_HISTORY);
		filterHistoryConfig.setExcludeWithoutHistory(excludeWithoutHistory);
		
    	Date dateFrom = DatasourceCommonBL.getDateFrom(parameters, locale);
    	Date dateTo = DatasourceCommonBL.getDateTo(parameters, locale);
    	/*Calendar calendar = Calendar.getInstance();
    	if (StringArrayParameterUtils.parseIntegerValue(parameters, DatasourceCommonBL.PARAMETER_NAME.SELECTED_PERIOD_TYPE, 
    			DatasourceCommonBL.PERIOD_TYPE.FROM_TO)==DatasourceCommonBL.PERIOD_TYPE.FROM_TO) {    	
	    	String strDateFrom = StringArrayParameterUtils.getStringValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DATE_FROM);	    	
	    	String strDateTo = StringArrayParameterUtils.getStringValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DATE_TO);	    	
	    	dateFrom = DateTimeUtils.getInstance().parseDateFromCalendarGUI(strDateFrom, locale);
	    	dateTo = DateTimeUtils.getInstance().parseDateFromCalendarGUI(strDateTo, locale);     		    	
    	} else {
    		int daysBefore = StringArrayParameterUtils.parseIntegerValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DAYS_BEFORE, 0);    		    		
    		dateTo = calendar.getTime();
    		calendar.add(Calendar.DATE, -daysBefore);
    		dateFrom = calendar.getTime();    		
    	}
    	if (dateFrom!=null) {
	    	calendar.setTime(dateFrom);
	    	CalendarUtil.clearTime(calendar);
	    	//the beginning of fromDate day
	    	dateFrom = calendar.getTime();
    	}
    	if (dateTo!=null) {
	    	calendar.setTime(dateTo);
	    	CalendarUtil.clearTime(calendar);
			calendar.add(Calendar.DATE, 1);
			//the end of toDate day (actually the beginning of the day after toDate)
			dateTo = calendar.getTime();
    	}*/
    	filterHistoryConfig.setDateFrom(dateFrom);
    	filterHistoryConfig.setDateTo(dateTo);		
    	filterHistoryConfig.setFields(StringArrayParameterUtils.parseIntegerArrValue(
    			parameters, PARAMETER_NAME.SELECTED_FIELDS));    	
    	filterHistoryConfig.setPerson(StringArrayParameterUtils.parseIntegerValue(
    			parameters, PARAMETER_NAME.SELECTED_PERSON));			
		return filterHistoryConfig;
	}	
}
