/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2010 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: OpenedClosedDatasource.java 2688 2010-11-04 16:47:10Z friedj $ 
 * 
 * Author: <Tamas Ruff> 
 * 
 */
package com.trackplus.plugin.report.openedClosed;

import java.io.OutputStream;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;

import com.aurel.track.beans.TPersonBean;
import com.aurel.track.dao.DAOFactory;
import com.aurel.track.dao.HistoryTransactionDAO;
import com.aurel.track.fieldType.constants.SystemFields;
import com.aurel.track.item.history.HistorySelectValues;
import com.aurel.track.plugin.DatasourceDescriptor;
import com.aurel.track.report.datasource.DatasourceCommonBL;
import com.aurel.track.report.datasource.IPluggableDatasource;
import com.aurel.track.report.export.ReportBeansToXML;
import com.aurel.track.resources.LocalizeUtil;
import com.aurel.track.util.CalendarUtil;
import com.aurel.track.util.DateTimeUtils;
import com.aurel.track.util.GeneralUtils;
import com.aurel.track.util.StringArrayParameterUtils;

public class OpenedClosedDatasource implements IPluggableDatasource {
	//Configuration page constants
	protected static interface CONFIGURATION_PARAMETERS {			    
	    static String STATUSES = "statuses";
		static String SELECTED_STATUS = "selectedStatus";		
		static String SELECTED_PROJECTRELEASE = "selectedProjectRelease";
	}

	private static Logger logger = Logger.getLogger(OpenedClosedDatasource.class);		
	private static HistoryTransactionDAO historyTransactionDAO = DAOFactory.getFactory().getHistoryTransactionDAO();
	
	/**
	 * Gets the data source object retrieved using the parameters settings
	 * @param parameters
	 * @param datasourceDescriptor
	 * @param personID the logged in person
	 * @param locale
	 * @return
	 */
	public Object getDatasource(Map parameters, DatasourceDescriptor datasourceDescriptor, 
			TPersonBean personBean, Locale locale) {
		
    	DateTimeUtils dateTimeUtils = DateTimeUtils.getInstance();
    	//get the workItemBeans
    	List workItemBeans = DatasourceCommonBL.getWorkItemBeans(parameters, personBean, locale);
    	if (workItemBeans==null || workItemBeans.size()==0) {
    		return null;
    	}
    	//get the workItemIDs
    	int[] workItemIDs = GeneralUtils.createIntArrFromIntegerList(GeneralUtils.createIntegerListFromBeanList(workItemBeans));

    	
    	//get the date period
    	Date dateFrom = null;
    	Date dateTo = null;
    	Calendar calendar = Calendar.getInstance();
    	if (StringArrayParameterUtils.parseIntegerValue(parameters, DatasourceCommonBL.PARAMETER_NAME.SELECTED_PERIOD_TYPE, DatasourceCommonBL.PERIOD_TYPE.FROM_TO)==DatasourceCommonBL.PERIOD_TYPE.FROM_TO) {    	
	    	String strDateFrom = StringArrayParameterUtils.getStringValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DATE_FROM);
	    	logger.debug("From date " + strDateFrom);
	    	String strDateTo = StringArrayParameterUtils.getStringValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DATE_TO);
	    	logger.debug("To date " + strDateTo);
	    	dateFrom = dateTimeUtils.parseGUIDate(strDateFrom, locale);
	    	dateTo = dateTimeUtils.parseGUIDate(strDateTo, locale);     		    	
    	} else {
    		int daysBefore = StringArrayParameterUtils.parseIntegerValue(parameters, DatasourceCommonBL.PARAMETER_NAME.DAYS_BEFORE, 0);
    		logger.debug("days before " + daysBefore);
    		dateTo = calendar.getTime();
    		calendar.add(Calendar.DATE, -daysBefore);
    		dateFrom = calendar.getTime();
    		
    	}
    	if (dateFrom!=null) {
	    	calendar.setTime(dateFrom);
	    	CalendarUtil.clearTime(calendar);
	    	//the beginning of fromDate day
	    	dateFrom = calendar.getTime();
    	}
    	if (dateTo!=null) {
	    	calendar.setTime(dateTo);
	    	CalendarUtil.clearTime(calendar);
			calendar.add(Calendar.DATE, 1);
			//the end of toDate day (actually the beginning of the day after toDate)
			dateTo = calendar.getTime();
    	}
    	//get the statusChange beans
    	//get the selected closed statuses 
    	List selectedStatusList = GeneralUtils.createIntegerListFromStringArr(
    			(String[])parameters.get(OpenedClosedDatasource.CONFIGURATION_PARAMETERS.SELECTED_STATUS));
    	if (selectedStatusList==null || selectedStatusList.size()==0) {
    		//if no status selected select all closed statuses
    		selectedStatusList = GeneralUtils.createIntegerListFromBeanList(OpenedClosedBL.loadClosedStates());
    	}
    	//List stateChangeBeansList = stateChangeDAO.loadForWorkItemsInTimeInterval(dateFrom, dateTo, workItemIDs, selectedStatusList);    	    	 	
    	List<HistorySelectValues> historySelectValuesList = historyTransactionDAO.getByWorkItemsFieldNewValuesDates(
    			workItemIDs, SystemFields.INTEGER_STATE, selectedStatusList, dateFrom, dateTo);
    	
    	//get the selected time interval
    	int selectedTimeInterval = StringArrayParameterUtils.parseIntegerValue(parameters, DatasourceCommonBL.PARAMETER_NAME.SELECTED_TIME_INTERVAL, DatasourceCommonBL.TIME_INTERVAL.MONTH);    		        		    	    
    	SortedMap openedClosedTimeSliceMap = new TreeMap();
    	OpenedClosedBL.openedClosedJavaBean(openedClosedTimeSliceMap, 
    			OpenedClosedBL.calculateNewWorkItems(dateFrom, dateTo, workItemBeans, selectedTimeInterval), true);
    	OpenedClosedBL.openedClosedJavaBean(openedClosedTimeSliceMap, 
    			OpenedClosedBL.calculateClosedWorkItems(dateFrom, dateTo, historySelectValuesList, selectedTimeInterval), false);
    	return OpenedClosedBL.convertToDOM(openedClosedTimeSliceMap.values(), locale, personBean.getFullName(), selectedTimeInterval);
    	
	   
	}
	
	/**
	 * Serializes the datasource in an XML file
	 */
	public void serializeDatasource(OutputStream outputStream,
			Object datasource) {
		ReportBeansToXML.convertToXml(outputStream, (Document)datasource);
	}
	
	/**
	 * Implements the serialization
	 */
	public boolean implementSerialization() {
		return true;
	}
	
	/**
	 * Prepares a map for rendering
	 * @param parameters
	 * @param datasourceDescriptor
	 * @param personID
	 * @param locale 
	 * @return
	 */
	public Map prepareParameters(Map parameters, DatasourceDescriptor datasourceDescriptor, 
			TPersonBean personBean, Locale locale) {
							
		if (parameters==null) {
			parameters = new HashMap();
		}
		
		DatasourceCommonBL.prepareCommonChartParameters(parameters, datasourceDescriptor, personBean, locale);
    	parameters.put(OpenedClosedDatasource.CONFIGURATION_PARAMETERS.STATUSES, 
    			LocalizeUtil.localizeDropDownList(OpenedClosedBL.loadClosedStates(), locale));
	    return parameters;
		
	}
	
	/**
	 * Get the names of those parameter names which should be saved in the session
	 * for future use of the same report configuration
	 * @return
	 */
	public List getSavedParameterNames() {
		List savedParameterNames = DatasourceCommonBL.getCommonSavedParameterNames();
		savedParameterNames.add(OpenedClosedDatasource.CONFIGURATION_PARAMETERS.SELECTED_STATUS);		
		return savedParameterNames;
	} 	
 	
}
