/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2010 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: StatusOverTimeBL.java 2694 2010-11-04 18:54:31Z friedj $ 
 * 
 * Author: <Tamas Ruff> 
 * 
 */
package com.trackplus.plugin.report.statusOverTime;

import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.aurel.track.beans.TStateBean;
import com.aurel.track.report.dashboard.StatusOverTimeGraph;
import com.aurel.track.util.DateTimeUtils;

public class StatusOverTimeBL {
	private static Logger logger = Logger.getLogger(StatusOverTimeBL.class);		
	
	static Document convertToDOM(SortedMap<Date, Map<Integer, Integer>> numberOfIssuesInStatusByDate, 
			Map<Integer, TStateBean> statusMap, String personName, int selectedTimeInterval, boolean newIssues) {		
		Document dom = null;
		try{
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
			DocumentBuilder builder = factory.newDocumentBuilder ();
			dom = builder.newDocument ();	
		}catch (FactoryConfigurationError e){
			logger.error ("Creating the DOM document failed with FactoryConfigurationError:" + e.getMessage());
			return null;
		}catch (ParserConfigurationException e){
			logger.error ("Creating the DOM document failed with ParserConfigurationException: " + e.getMessage());
			return null;
		}
		Element root = dom.createElement ("track-report");
        Element createdBy =createDomElement("createdBy",personName,dom);
        root.appendChild(createdBy);    
        Element timeInterval =createDomElement("timeInterval",String.valueOf(selectedTimeInterval),dom);
        root.appendChild(timeInterval);    
        Iterator<Date> iterator = numberOfIssuesInStatusByDate.keySet().iterator();
        while (iterator.hasNext()) {
        	Date date = iterator.next();
        	Map<Integer, Integer> numberInStateMap = numberOfIssuesInStatusByDate.get(date); 
        	if (numberInStateMap!=null && numberInStateMap.size()>0) {
        		Element period = dom.createElement ("period");            
              	period.appendChild(createDomElement("date", DateTimeUtils.formatISODateTime(date), dom));
              	Iterator<Integer> itrStatus = numberInStateMap.keySet().iterator();              	              	
              	if (newIssues) {
              		Integer noOfNewIssues = numberInStateMap.get(StatusOverTimeGraph.ENTITY_PLACEHOLDER);
              		if (noOfNewIssues==null) {
              			period.appendChild(createDomElement("newIssues", "0", dom));
              		} else {
              			period.appendChild(createDomElement("newIssues", noOfNewIssues.toString(), dom));
              		}
              	} else {
              		Element issuesInStatus = dom.createElement ("issuesInStatus");
                  	period.appendChild(issuesInStatus);
              		while (itrStatus.hasNext()) {
    					Integer statusID = itrStatus.next();
    					Integer numberOfIssues = numberInStateMap.get(statusID);					
    					TStateBean stateBean = statusMap.get(statusID);
    					if (numberOfIssues!=null && stateBean!=null) {
    						String statusLabel = stateBean.getLabel();
    						Element element = createDomElement("s"+stateBean.getObjectID().toString(), numberOfIssues.toString(), dom);
    						element.setAttribute("label", statusLabel); 
    						period.appendChild(element);
    					}
    				}
              	}
              	
              	root.appendChild(period);
        	}            
		}
		dom.appendChild (root);
        return dom;
	}
	
	/**
	 * @param elementName
	 * @param elementValue
	 * @param dom
	 * @return
	 */
	private static Element createDomElement (String elementName, String elementValue, Document dom) {
		Element element = dom.createElement(elementName.replaceAll("\\s", ""));
		if (elementValue == null || "".equals(elementValue.trim())) {
			element.appendChild (dom.createTextNode (""));
		} else {
			element.appendChild (dom.createTextNode(elementValue));
		}						
		return element;
	}	
}
