/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2010 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: StatusOverTimeDatasource.java 2694 2010-11-04 18:54:31Z friedj $ 
 * 
 * Author: <Tamas Ruff> 
 * 
 */
package com.trackplus.plugin.report.statusOverTime;

import java.io.OutputStream;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;

import com.aurel.track.beans.TPersonBean;
import com.aurel.track.beans.TStateBean;
import com.aurel.track.dao.DAOFactory;
import com.aurel.track.dao.StateDAO;
import com.aurel.track.plugin.DatasourceDescriptor;
import com.aurel.track.report.dashboard.StatusOverTimeGraph;
import com.aurel.track.report.datasource.DatasourceCommonBL;
import com.aurel.track.report.datasource.IPluggableDatasource;
import com.aurel.track.report.export.ReportBeansToXML;
import com.aurel.track.resources.LocalizeUtil;
import com.aurel.track.util.CalendarUtil;
import com.aurel.track.util.DateTimeUtils;
import com.aurel.track.util.GeneralUtils;
import com.aurel.track.util.StringArrayParameterUtils;

public class StatusOverTimeDatasource implements IPluggableDatasource {
	private static StateDAO stateDAO = DAOFactory.getFactory().getStateDAO();
	//Configuration page constants
	protected static interface CONFIGURATION_PARAMETERS {			    
	    static String STATUSES = "statuses";
		static String SELECTED_STATUS = "selectedStatus";
		static String CALCULATION_MODES = "calculationModes";
		static String SELECTED_CALCULATION_MODE = "selectedCalculationMode";
		static String SELECTED_PROJECTRELEASE = "selectedProjectRelease";
	}

	private static Logger logger = Logger.getLogger(StatusOverTimeDatasource.class);		
	
	/**
	 * Gets the data source object retrieved using the parameters settings
	 * @param configParameters
	 * @param datasourceDescriptor
	 * @param personID the logged in person
	 * @param locale
	 * @return
	 */
	public Object getDatasource(Map configParameters, DatasourceDescriptor datasourceDescriptor, 
			TPersonBean personBean, Locale locale) {		    	    
		List<Integer> statusIDs;
    	Integer[] statusIDsArr =  StringArrayParameterUtils.parseIntegerArrValue(configParameters, CONFIGURATION_PARAMETERS.SELECTED_STATUS);
    	if (statusIDsArr==null || statusIDsArr.length==0) {
    		 statusIDs = GeneralUtils.createIntegerListFromBeanList(stateDAO.loadAll());
    	} else {    		
    		statusIDs = GeneralUtils.createListFromIntArr(statusIDsArr);    		
    	}
    	Date dateFrom = null;
    	Date dateTo = null;
    	Calendar calendar = Calendar.getInstance();
    	if (StringArrayParameterUtils.parseIntegerValue(configParameters, StatusOverTimeGraph.CONFIGURATION_PARAMETERS.SELECTED_PERIOD_TYPE, 
    			StatusOverTimeGraph.PERIOD_TYPE.FROM_TO)==StatusOverTimeGraph.PERIOD_TYPE.FROM_TO) {    	
	    	String strDateFrom = StringArrayParameterUtils.getStringValue(configParameters, StatusOverTimeGraph.CONFIGURATION_PARAMETERS.DATE_FROM);
	    	logger.debug("From date " + strDateFrom);
	    	String strDateTo = StringArrayParameterUtils.getStringValue(configParameters, StatusOverTimeGraph.CONFIGURATION_PARAMETERS.DATE_TO);
	    	logger.debug("To date " + strDateTo);
	    	dateFrom = DateTimeUtils.getInstance().parseGUIDate(strDateFrom, locale);
	    	dateTo = DateTimeUtils.getInstance().parseGUIDate(strDateTo, locale);     		    	
    	} else {
    		int daysBefore = StringArrayParameterUtils.parseIntegerValue(configParameters, StatusOverTimeGraph.CONFIGURATION_PARAMETERS.DAYS_BEFORE, 0);    		    		
    		dateTo = calendar.getTime();
    		calendar.add(Calendar.DATE, -daysBefore);
    		dateFrom = calendar.getTime();    		
    	}
    	if (dateFrom!=null) {
	    	calendar.setTime(dateFrom);
	    	CalendarUtil.clearTime(calendar);
	    	//the beginning of fromDate day
	    	dateFrom = calendar.getTime();
    	}
    	if (dateTo!=null) {
	    	calendar.setTime(dateTo);
	    	CalendarUtil.clearTime(calendar);
			calendar.add(Calendar.DATE, 1);
			//the end of toDate day (actually the beginning of the day after toDate)
			dateTo = calendar.getTime();
    	}    	    
    	int selectedTimeInterval = StringArrayParameterUtils.parseIntegerValue(configParameters, 
    			StatusOverTimeGraph.CONFIGURATION_PARAMETERS.SELECTED_TIME_INTERVAL, StatusOverTimeGraph.TIME_INTERVAL.MONTH);
    	int selectedCalculationMode = StringArrayParameterUtils.parseIntegerValue(configParameters, 
    			StatusOverTimeGraph.CONFIGURATION_PARAMETERS.SELECTED_CALCULATION_MODE, StatusOverTimeGraph.CALCULATION_MODE.NEW);    	
    	    	
    	SortedMap<Integer, SortedMap<Integer, Map<Integer, Integer>>> yearToPeriodToEntityIDToNumbersMap = 
    		new TreeMap<Integer, SortedMap<Integer, Map<Integer, Integer>>>();
    	List workItemBeans = DatasourceCommonBL.getWorkItemBeans(configParameters, personBean, locale);
    	if (workItemBeans==null || workItemBeans.size()==0) {
    		return null;
    	}
    	int[] workItemIDs = GeneralUtils.createIntArrFromIntegerArr(GeneralUtils.getBeanIDs(workItemBeans));
    	boolean newIssues = false;
    	switch (selectedCalculationMode) {
    	case StatusOverTimeGraph.CALCULATION_MODE.NEW:
    		yearToPeriodToEntityIDToNumbersMap = StatusOverTimeGraph.calculateNewWorkItems(workItemBeans,
    				dateFrom, dateTo, selectedTimeInterval);
    		newIssues = true;
    		break;
    	case StatusOverTimeGraph.CALCULATION_MODE.ACTUAL_SAMPLE:
    		yearToPeriodToEntityIDToNumbersMap = StatusOverTimeGraph.calculateTotalInStatus(workItemIDs,
    				dateFrom, dateTo, statusIDs, selectedTimeInterval, locale);    	     		
    		break;
    	case StatusOverTimeGraph.CALCULATION_MODE.ACTUAL_ACTIVITY:
    		yearToPeriodToEntityIDToNumbersMap = StatusOverTimeGraph.calculateStatus(workItemIDs,
    				dateFrom, dateTo, statusIDs, selectedTimeInterval, locale);    	    		
    		break;
    	case StatusOverTimeGraph.CALCULATION_MODE.ACCUMULATED_ACTIVITY:
    		yearToPeriodToEntityIDToNumbersMap = StatusOverTimeGraph.calculateStatus(workItemIDs,
    				dateFrom, dateTo, statusIDs, selectedTimeInterval, locale);    	    		    	
    		break;
    	}
		
    	Map<Integer, TStateBean> statusMap = GeneralUtils.createMapFromList(LocalizeUtil.localizeDropDownList(stateDAO.loadAll(), locale));    	
    	SortedMap numberOfIssuesInStatusByDate = DatasourceCommonBL.transformPeriodsToDates(yearToPeriodToEntityIDToNumbersMap, selectedTimeInterval);
    	    	  		        		    	    
    	return StatusOverTimeBL.convertToDOM(numberOfIssuesInStatusByDate, statusMap, personBean.getFullName(), selectedTimeInterval, newIssues);
    		  
	}
	
	/**
	 * Serializes the datasource in an XML file
	 */
	public void serializeDatasource(OutputStream outputStream,
			Object datasource) {
		ReportBeansToXML.convertToXml(outputStream, (Document)datasource);
	}
	
	/**
	 * Implements the serialization
	 */
	public boolean implementSerialization() {
		return true;
	}
	
	/**
	 * Prepares a map for rendering
	 * @param parameters
	 * @param datasourceDescriptor
	 * @param personID
	 * @param locale 
	 * @return
	 */
	public Map prepareParameters(Map parameters, DatasourceDescriptor datasourceDescriptor, 
			TPersonBean personBean, Locale locale) {						
		if (parameters==null) {
			parameters = new HashMap();
		}		
		DatasourceCommonBL.prepareCommonChartParameters(parameters, datasourceDescriptor, personBean, locale);
    	parameters.put(StatusOverTimeDatasource.CONFIGURATION_PARAMETERS.STATUSES, 
    			LocalizeUtil.localizeDropDownList(stateDAO.loadAll(), locale));    	
    	parameters.put(StatusOverTimeDatasource.CONFIGURATION_PARAMETERS.CALCULATION_MODES, 
    			StatusOverTimeGraph.possibleCalculationModes(locale, datasourceDescriptor.getBundleName()));    	    	
	    return parameters;
		
	}	
	
	/**
	 * Get the names of those parameter names which should be saved in the session
	 * for future use of the same report configuration
	 * @return
	 */
	public List getSavedParameterNames() {
		List savedParameterNames = DatasourceCommonBL.getCommonSavedParameterNames();
		savedParameterNames.add(StatusOverTimeDatasource.CONFIGURATION_PARAMETERS.SELECTED_STATUS);		
		savedParameterNames.add(StatusOverTimeDatasource.CONFIGURATION_PARAMETERS.SELECTED_CALCULATION_MODE);
		return savedParameterNames;
	} 	
 			
}
